#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include "CisModuleU.h"


//main function to all the operation
int main (int argc, char *argv[]) 
{
	//get the default values
	if(argc==1)
	{
		PrintReadme();
		return 0;
	}

	INPUTPARAMETER* pInputParam=new INPUTPARAMETER;
	SetDefaultParam(pInputParam);
	ParseInputParam(pInputParam,argc,argv);

	//exit if w<5 or W>40
	if(pInputParam->nMinWidth<5 || pInputParam->nMaxWidth>40)
	{
		printf("input motif width range error!\n");
		exit(0);
	}
	//eixt if K<0
	if(pInputParam->nMotifNumber<1)
	{
		printf("Input number of transcription factors error!\n");
		exit(0);
	}
	//exit if L<w
	if(pInputParam->nModuleLength<pInputParam->nMinWidth)
	{
		printf("Input module length too small!\n");
		exit(0);
	}

	for(int r=0;r<pInputParam->nRun;r++)
		RunModSampling(pInputParam);
	//delete input param
	delete pInputParam;

	return 0;
}

//set default p
void SetDefaultParam(INPUTPARAMETER* pInputParam)
{
	pInputParam->bReverse=true;			
	pInputParam->nMaxIteration=2000;
	pInputParam->nMotifNumber=3;
	pInputParam->nMinWidth=10;
	pInputParam->nMaxWidth=14;
	pInputParam->nModuleLength=200;
	pInputParam->bMotifMode=false;
	pInputParam->bFixModuleLength=true;
	pInputParam->bSampleParameters=false;
	pInputParam->strInputSeq="";
	pInputParam->strInputBgPr="";
	pInputParam->strInputPWM="";
	pInputParam->bInitialPWM=false;
	pInputParam->bInputBackground=false;
	pInputParam->strOutput="Output.txt";	
	pInputParam->nRun=1;
	pInputParam->bISBF=false;
	pInputParam->dblMotifCutoff=0.5;
}

//parse input parameters for the program
void ParseInputParam(INPUTPARAMETER* pInputParam, int argc, char* argv[])
{
	pInputParam->strInformation="CisModule (compiled on 12/5/2005)\n";
	for(int i=0;i<argc;i++)
	{
		pInputParam->strInformation=pInputParam->strInformation+argv[i]+" ";
		if(argv[i][0]=='-')
		{
			if(argv[i][1]=='i')
			{
				pInputParam->strInputSeq=argv[i+1];
			}
			else if(argv[i][1]=='o')
			{
				pInputParam->strOutput=argv[i+1];
			}
			else if(argv[i][1]=='n')
			{
				pInputParam->nMaxIteration=atoi(argv[i+1]);
			}
			else if(argv[i][1]=='N')
			{
				pInputParam->nRun=atoi(argv[i+1]);
			}
			else if(argv[i][1]=='r')
			{
				if(argv[i+1][0]=='0') pInputParam->bReverse=false;
				else pInputParam->bReverse=true;
			}
			else if(argv[i][1]=='K')
			{
				pInputParam->nMotifNumber=atoi(argv[i+1]);
			}
			else if(argv[i][1]=='L')
			{
				pInputParam->nModuleLength=atoi(argv[i+1]);
			}
			else if(argv[i][1]=='w')
			{
				pInputParam->nMinWidth=atoi(argv[i+1]);
			}
			else if(argv[i][1]=='W')
			{
				pInputParam->nMaxWidth=atoi(argv[i+1]);
			}
			else if(argv[i][1]=='b')
			{
				pInputParam->bInputBackground=true;
				pInputParam->strInputBgPr=argv[i+1];
			}
			else if(argv[i][1]=='P')
			{
				pInputParam->bSampleParameters=true;
			}
			else if(argv[i][1]=='s')
			{
				pInputParam->bMotifMode=true;
			}
			else if(argv[i][1]=='d')
			{
				pInputParam->bFixModuleLength=false;
			}
			else if(argv[i][1]=='B')
			{
				pInputParam->bISBF=true;
			}
			else if(argv[i][1]=='c')
			{
				pInputParam->dblMotifCutoff=atof(argv[i+1]);
			}
			else if(argv[i][1]=='M')
			{
				pInputParam->bInitialPWM=true;
				pInputParam->strInputPWM=argv[i+1];
			}
		}
	}
}

//one run of the module sampler
void RunModSampling(INPUTPARAMETER* pInputParam)
{
	CGibbsDP ModSampler;

	//get the input parameters
	ModSampler.m_strInformation=pInputParam->strInformation;
	ModSampler.m_strInputSeq=pInputParam->strInputSeq;
	ModSampler.m_strOutput=pInputParam->strOutput;
	ModSampler.m_nMaxIteration=pInputParam->nMaxIteration;
	ModSampler.m_nBI=int(ModSampler.m_nMaxIteration*0.7);
	ModSampler.m_bReverse=pInputParam->bReverse;
	ModSampler.m_bMotifMode=pInputParam->bMotifMode;
	ModSampler.m_nMotifNumber=pInputParam->nMotifNumber;
	ModSampler.m_nModuleLength=pInputParam->nModuleLength;
	ModSampler.m_nMinWidth=pInputParam->nMinWidth;
	ModSampler.m_nMaxWidth=pInputParam->nMaxWidth;
	ModSampler.m_bFixMLength=pInputParam->bFixModuleLength;
	ModSampler.m_bOutputBF=pInputParam->bISBF;
	ModSampler.m_dblMotifCutoff=pInputParam->dblMotifCutoff;
	ModSampler.m_bSampleParameters=pInputParam->bSampleParameters;
	ModSampler.m_bInputBackground=pInputParam->bInputBackground;
	ModSampler.m_strInputBgfile=pInputParam->strInputBgPr;
	ModSampler.m_strPWMFile=pInputParam->strInputPWM;
	ModSampler.m_bInitalPWM=pInputParam->bInitialPWM;
	
	//sampling by DA
	ModSampler.DataAugmentation();

}

void PrintReadme()
{
	printf("\n******************************\n");
	printf("   CisModule 1.0 Dec. 5 2005.\n");
	printf("******************************\n");
	printf("\nUsage: ./CisModule -i Seqfile (options)\n");
	printf("Seqfile must be in FASTA format and each line in the file should contain no more than 10K characters\n\n");
	printf("Options:\n");
	printf("\t-o\tOutput file (default output.txt)\n");
	printf("\t-n\tMaximum number of iterations (default 2000)\n");
	printf("\t-c\tThreshold for motif prediction (between 0 and 1; default 0.5)\n");
	printf("\t\tYou may increase this parameter if CisModule predicts many sites\n");
	printf("\t-r\t1: search for both strands; 0: only search for forward strand (default 1)\n");
	printf("\t-K\tNumber of transcription factors (default 3)\n");
	printf("\t-L\tModule length (default 200)\n");
	printf("\t-w\tMinimum width for motifs (default 10); should be >= 5 \n");
	printf("\t-W\tMaximum width for motifs (default 14); should be <= 40 \n");
	printf("\t-P\tSample parameters each iteration. See readme file for details\n");
	printf("\t-B\tOutput log-posterior model odds over null model (pure background)\n");
	printf("\t-s\tDegerate to search for K independent motifs without considering module structure\n");
}

