#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include "GibbsDP.h"
#include "StochasticDistribution.h"

#define MAXWIDTH	50
#define MAXSITENUMBER	10000

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGibbsDP::CGibbsDP()
{
	m_nMotifNumber=1;			//# of motifs: K
	m_nModuleLength=200;		//length of each module: l
	m_nSequenceNumber=0;		//number of sequences
	m_dblOptLogJPP=-1000;		//optimal log posterior probability
	m_bReverse=true;			//sample backward or not
	m_nMaxWidth=20;				//maximal allowable width
	m_nMinWidth=8;				//minimal allowable width

	m_pSiteHead=NULL;			//head pointer to each kind of motif site
	m_pModule=NULL;				//modules for each sequence
	m_pnWeightMat=NULL;			//weight matrices for all the motifs
	m_pnMotifWidth=NULL;		//width for each motif
	m_pnSiteCount=NULL;			//number of site for each motif

	m_pstrSequence=NULL;		//sets of input sequences
	m_pstrSeqName=NULL;			//names of the sequences ">****"
	m_pOptSiteHead=NULL;		//optimal sample

	m_pnPseudoSiteCounts=NULL;		// \alpha(0,1,...,K)

	m_bMotifMode=false;		//degenerate to site sampler

	m_bPriorWM=false;			//have prior information for weight matrix
	m_pnInitialMat=NULL;		//initial weight matrices for motifs

	m_bReverse=true;			//search reverse strand
	m_bFixWidth=false;
	m_bFixMLength=false;
	m_nDeltL=10;
	m_nModuleLength=-1;
	m_bStopRun=false;
	m_bOverMaxSite=false;
}

CGibbsDP::~CGibbsDP()
{
	FreeMemery();
}

void CGibbsDP::FreeMemery()
{
	if(m_pstrSequence!=NULL)
	{
		delete[] m_pstrSequence;
		m_pstrSequence=NULL;
	}
	if(m_pstrSeqName!=NULL)
	{
		delete[] m_pstrSeqName;
		m_pstrSeqName=NULL;
	}
	if(m_pnMotifWidth!=NULL)
	{
		delete[] m_pnMotifWidth;
		m_pnMotifWidth=NULL;
	}
	if(m_pnSiteCount!=NULL)
	{
		delete[] m_pnSiteCount;
		m_pnSiteCount=NULL;
	}
	if(m_pnPseudoSiteCounts!=NULL)
	{
		delete[] m_pnPseudoSiteCounts;
		m_pnPseudoSiteCounts=NULL;
	}
	if(m_pnWeightMat!=NULL)
	{
		for(int k=0;k<m_nMotifNumber;k++)
		{
			if(m_pnWeightMat[k]!=NULL)
			{
				delete[] m_pnWeightMat[k];
				m_pnWeightMat[k]=NULL;
			}
		}
		delete[] m_pnWeightMat;
		m_pnWeightMat=NULL;
	}
	if(m_pModule!=NULL)
	{
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			if(m_pModule[n].pnStart!=NULL)
			{
				delete[] m_pModule[n].pnStart;
				m_pModule[n].pnStart=NULL;
			}
		}
		delete[] m_pModule;
		m_pModule=NULL;
	}
	if(m_pSiteHead!=NULL)
	{
		for(int k=0;k<m_nMotifNumber;k++)
		{
			DelSiteHead(m_pSiteHead[k]);
		}
		delete[] m_pSiteHead;
	}

	if(m_pOptSiteHead!=NULL)
	{
		for(int k=0;k<m_nMotifNumber;k++)
		{
			DelSiteHead(m_pOptSiteHead[k]);
		}
		delete[] m_pOptSiteHead;
	}

	//initial prior information
	if(m_pnInitialMat!=NULL)		//initial weight matrices for motifs
	{
		for(int k=0;k<m_nMotifNumber-1;k++)
		{
			if(m_pnInitialMat[k]!=NULL)
			{
				delete[] m_pnInitialMat[k];
				m_pnInitialMat[k]=NULL;
			}
		}
		delete[] m_pnInitialMat;
		m_pnInitialMat=NULL;
	}
}

//delete sitehead of one motif
void CGibbsDP::DelSiteHead(SITEINFO** pSiteHead)		
{
	if(pSiteHead!=NULL)
	{
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			SITEINFO* pCur=pSiteHead[n];
			SITEINFO* pDel;
			while(pCur)
			{
				pDel=pCur;
				pCur=pCur->pNextSite;
				delete pDel;
			}
		}
		delete[] pSiteHead;
	}
}

////filter sites by their nMA
void CGibbsDP::FilterSite(SITEINFO** pSiteHead,double dblThreshold)	
{
	if(pSiteHead!=NULL)
	{
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			SITEINFO* pCur=pSiteHead[n];
			SITEINFO* pDel;
			while(pCur->pNextSite)
			{
				pDel=pCur->pNextSite;
				if(pDel->nMA<dblThreshold)
				{
					pCur->pNextSite=pDel->pNextSite;
					delete pDel;
				}
				else
					pCur=pCur->pNextSite;
			}
		}
	}	
}


void CGibbsDP::DataAugmentation()
{
	m_dblOptLogJPP=-1e100;
	srand( (unsigned)time( NULL ) );
	Initialize();
	BuildBackground();

	//print some information
	printf(m_strInformation.m_pString);
	printf("\nInput %d sequences.\n",m_nSequenceNumber);

	double dblOptMAPLHK=-1e100;
	RandPickSite();

	//definition of parameters
	double r;
	double* q=new double[m_nMotifNumber+1];
	double** Theta=new double*[m_nMotifNumber];
	int k=0;
	for(k=0;k<m_nMotifNumber;k++)
		Theta[k]=new double[m_pnMotifWidth[k]*4];

	//marginal distributions
	int** pnMA=new int*[m_nSequenceNumber];
	int*  pnMW=new int[m_nMotifNumber*MAXWIDTH];
	int*  pnMC=new int[m_nMotifNumber*MAXSITENUMBER];
	int** pnMAMod=new int*[m_nSequenceNumber];
	int pnML[50];
	SetZerosMD(pnMA,pnMW,pnMC,pnMAMod,&(pnML[0]));

	//get sequence lengths and the probability of positions
	int* pnSeqLength=new int[m_nSequenceNumber];
	double** pdblLogPrPos=new double*[m_nSequenceNumber];
	int nAvgSeqLength=0;
	int n=0;
	for(n=0;n<m_nSequenceNumber;n++)
	{
		pnSeqLength[n]=m_pstrSequence[n].GetLength();
		nAvgSeqLength+=pnSeqLength[n];
		pdblLogPrPos[n]=new double[pnSeqLength[n]*(m_nMotifNumber*2+1)];
	}
	nAvgSeqLength/=m_nSequenceNumber;

	//calculate background probabilities in log:
	CalLogBgPrPos(pdblLogPrPos,pnSeqLength);

	int nIteration=0;
	while(nIteration<m_nMaxIteration)
	{
		//phase shift
		if(nIteration<m_nBI && nIteration%50==49)
		{
			int k=0;
			for(k=0;k<m_nMotifNumber;k++)
				UpdateWidth(k);

			for(k=0;k<m_nMotifNumber;k++)
				PhaseShift(k);

			for(k=0;k<m_nMotifNumber;k++)
			{
				delete[] Theta[k];
				Theta[k]=new double[m_pnMotifWidth[k]*4];
			}
		} 
		//sampling parameters
		if(m_bSampleParameters)
			SampleParameters(&r,q,Theta);
		else
			UpdateParameters(&r,q,Theta);

		//initial PWMs
		if(nIteration==0 && m_bInitalPWM)
		{
			ReadInputPWM(Theta);
		}

		//sampling module
		//propose to change module length
		int nMLPropIndex=-1;
		int nMaxModuleLength=500;
		if(nMaxModuleLength>nAvgSeqLength) nMaxModuleLength=nAvgSeqLength;
		if(nIteration>m_nBI && !m_bFixMLength)
		{
			nMLPropIndex=rand()%4;	//0: left extent; 1: right extent; 2: left shrink; 3: right shrink
			if(m_nModuleLength<=80 && nMLPropIndex>=2) nMLPropIndex=-1;
			else if(m_nModuleLength>=nMaxModuleLength && nMLPropIndex<2) nMLPropIndex=-1;
		}

		CalLogPrPos(pdblLogPrPos,pnSeqLength,Theta);

		m_dblMAPLHK=0;
		for(n=0;n<m_nSequenceNumber;n++)
			ApproxSampleCurSeq(n,r,q,Theta,pdblLogPrPos[n],&nMLPropIndex); 

		//sample module length by Metropolis update
		if(nIteration>m_nBI && !m_bFixMLength && nMLPropIndex>=0)
			UpdateModuleLength(nMLPropIndex);

		//sample sites
		for(n=0;n<m_nSequenceNumber;n++)
		{
			int nModuleLength=m_nModuleLength;
			if(m_nModuleLength>pnSeqLength[n]) nModuleLength=pnSeqLength[n];

			int nCount=m_pModule[n].nCount;
			for(int m=0;m<nCount;m++)
			{
				int nEnd=m_pModule[n].pnStart[m]+nModuleLength-1;
				SampleSites(q,pdblLogPrPos[n],n,nEnd,nModuleLength);
			}
		}

		//calculate weight matrix and scores
		double dblCurScore=CalLogPostPrSample();
		if(m_dblOptLogJPP<dblCurScore)
		{
			CopySample(m_pSiteHead,m_pOptSiteHead);
			m_dblOptLogJPP=dblCurScore;
		}

		if(m_dblMAPLHK>dblOptMAPLHK)
			dblOptMAPLHK=m_dblMAPLHK;


		nIteration++;

		//record the marginal distribution
		if(nIteration>m_nBI)
			RecordMD(pnMA,pnMW,pnMC,pnMAMod,pnML);

		//output some information
		if(nIteration%50==0)		OutputInfo(nIteration);

	}

	//using the marginal distributions to generate predicted sites
	SumNeighborMD(pnMA);
	InferSiteCount(pnMC);
	InferModule(pnMAMod,pnSeqLength);
	FindOptimum(pnMA,pnMW,m_nMaxIteration-m_nBI+1,true);
	AlignSites(pnMA);

	if(m_bOutputBF)
	{
		m_pnSiteCount[m_nMotifNumber]=m_nModuleCount*m_nModuleLength;
		for(int k=0;k<m_nMotifNumber;k++)
		{
			m_pnSiteCount[m_nMotifNumber]-=m_pnSiteCount[k];
		}
		if(m_pnSiteCount[m_nMotifNumber]<0) m_pnSiteCount[m_nMotifNumber]=0;
		ISforBF(&r,q,Theta,pnSeqLength);
	}

	Output(m_strOutput,pnMA,pnMAMod,pnSeqLength);
	OutputModPr(pnMAMod,pnSeqLength,m_nMaxIteration-m_nBI+1,m_strOutput);

//	OutputSite();

	DeleteMD(pnMA,pnMW,pnMC,pnMAMod);

	delete[] q;
	for(k=0;k<m_nMotifNumber;k++)
		delete[] Theta[k];
	delete[] Theta;
	for(n=0;n<m_nSequenceNumber;n++)
	{
		delete[] pdblLogPrPos[n];
	}
	delete[] pdblLogPrPos;
	delete[] pnSeqLength;

}

void CGibbsDP::Initialize()
{
	ReadInputSeq();
	//motif width
	m_pnMotifWidth=new int[m_nMotifNumber];
	int k=0;
	for(k=0;k<m_nMotifNumber;k++)
	{
		double dblU=double(rand()+1.0)/(RAND_MAX+2.0);
		m_pnMotifWidth[k]=m_nMinWidth+int((m_nMaxWidth-m_nMinWidth+1)*dblU);
		if(m_pnMotifWidth[k]<m_nMinWidth) m_pnMotifWidth[k]=m_nMinWidth;
		if(m_pnMotifWidth[k]>m_nMaxWidth) m_pnMotifWidth[k]=m_nMaxWidth;		
	}	


	//weight matrix
	m_pnWeightMat=new int*[m_nMotifNumber];
	for(k=0;k<m_nMotifNumber;k++)
	{
		m_pnWeightMat[k]=new int[4*m_pnMotifWidth[k]];
	}

	//number of sites for each motif: including background in module
	m_pnSiteCount=new int[m_nMotifNumber+1];

	//module location
	m_pModule=new MODULEINFO[m_nSequenceNumber];
	int n=0;
	for(n=0;n<m_nSequenceNumber;n++)
	{
		m_pModule[n].nCount=0;
		m_pModule[n].pnStart=NULL;
		m_pModule[n].dblLogModScore=0;
	}

	//heads of the motifs on each sequence
	m_pSiteHead=new SITEINFO**[m_nMotifNumber];
	m_pOptSiteHead=new SITEINFO**[m_nMotifNumber];
	for(k=0;k<m_nMotifNumber;k++)
	{
		m_pSiteHead[k]=new SITEINFO*[m_nSequenceNumber];
		m_pOptSiteHead[k]=new SITEINFO*[m_nSequenceNumber];
		for(n=0;n<m_nSequenceNumber;n++)
		{
			m_pSiteHead[k][n]=new SITEINFO;
			m_pSiteHead[k][n]->pNextSite=NULL;
			m_pOptSiteHead[k][n]=new SITEINFO;
			m_pOptSiteHead[k][n]->pNextSite=NULL;
		}
	}

	//prior: pseudoCounts
	//1) Module counts
	m_pnPseudoModuleCounts[0]=1;
	m_pnPseudoModuleCounts[1]=1;

	//2) site counts
	m_pnPseudoSiteCounts=new int[m_nMotifNumber+1];
	for(k=0;k<=m_nMotifNumber;k++)
		m_pnPseudoSiteCounts[k]=1;

	//3) basepair counts
	for(int j=0;j<4;j++)
		m_pnPseudoBpCounts[j]=1;

	if(m_bMotifMode) 
	{
		m_nModuleLength=m_nTotalLength;
		m_bFixMLength=true;
	}

}

void CGibbsDP::SampleParameters(double* pr,double* q,double** Theta)
{
	CStochasticDistribution RandNumber;
	int L_n=m_nTotalLength;
	int M_n=CountModule(m_pModule);
	if(!m_bMotifMode)
		*pr=RandNumber.Beta(M_n+m_pnPseudoModuleCounts[0],L_n-M_n*m_nModuleLength+m_pnPseudoModuleCounts[1]);
	else
		*pr=1-1e-10;

	//2) q_n
	int* pnSCount=new int[m_nMotifNumber+1];
	int nMBCount=M_n*m_nModuleLength;
	if(m_bMotifMode) nMBCount=m_nTotalLength;
	int k=0;
	for(k=0;k<m_nMotifNumber;k++)
	{
		nMBCount-=m_pnSiteCount[k]*m_pnMotifWidth[k];
	}
	pnSCount[m_nMotifNumber]=nMBCount+m_pnPseudoSiteCounts[m_nMotifNumber];
	for(k=0;k<m_nMotifNumber;k++)
	{
		pnSCount[k]=m_pnSiteCount[k]+m_pnPseudoSiteCounts[k];
	}		
	RandNumber.Dirichlet(pnSCount,m_nMotifNumber+1,q);
	delete[] pnSCount;


	//calculate theta_n
	int* pnBpCount=new int[4];
	for(k=0;k<m_nMotifNumber;k++)
	{
		for(int i=0;i<m_pnMotifWidth[k];i++)
		{
			for(int j=0;j<4;j++)
			{
				pnBpCount[j]=m_pnWeightMat[k][i*4+j]+m_pnPseudoBpCounts[j];
			}
			RandNumber.Dirichlet(pnBpCount,4,&(Theta[k][i*4]));
		}
	}
	delete[] pnBpCount;

}

void CGibbsDP::UpdateParameters(double* pr,double* q,double** Theta)
{
	CStochasticDistribution RandNumber;
	int L_n=m_nTotalLength;
	int M_n=CountModule(m_pModule);
	if(!m_bMotifMode)
		*pr=double(M_n+m_pnPseudoModuleCounts[0])/(L_n-M_n*m_nModuleLength+M_n+m_pnPseudoModuleCounts[0]+m_pnPseudoModuleCounts[1]);
	else
		*pr=1-1e-10;

	//2) q_n
	int* pnSCount=new int[m_nMotifNumber+1];
	int nMBCount=M_n*m_nModuleLength;
	if(m_bMotifMode) nMBCount=m_nTotalLength;
	int k=0;
	for(k=0;k<m_nMotifNumber;k++)
	{
		nMBCount-=m_pnSiteCount[k]*m_pnMotifWidth[k];
	}
	pnSCount[m_nMotifNumber]=nMBCount+m_pnPseudoSiteCounts[m_nMotifNumber];
	int nTotalCount=pnSCount[m_nMotifNumber];
	for(k=0;k<m_nMotifNumber;k++)
	{
		pnSCount[k]=m_pnSiteCount[k]+m_pnPseudoSiteCounts[k];
		nTotalCount+=pnSCount[k];
	}		
	for(k=0;k<m_nMotifNumber+1;k++)
		q[k]=double(pnSCount[k])/nTotalCount;
	delete[] pnSCount;


	//calculate theta_n
	int* pnBpCount=new int[4];
	for(k=0;k<m_nMotifNumber;k++)
	{
		for(int i=0;i<m_pnMotifWidth[k];i++)
		{
			int nTotalSite=0;
			int j=0;
			for(j=0;j<4;j++)
			{
				pnBpCount[j]=m_pnWeightMat[k][i*4+j]+m_pnPseudoBpCounts[j];
				nTotalSite+=pnBpCount[j];
			}
			for(j=0;j<4;j++)
			Theta[k][i*4+j]=double(pnBpCount[j])/nTotalSite;
		}
	}
	delete[] pnBpCount;
}

//output information
void CGibbsDP::OutputInfo(int nIteration)		//output some information
{
	CString strInfofile;
	strInfofile=m_strOutput+"_info.txt";
	
	FILE* file=fopen(strInfofile.m_pString,"a+");
	fprintf(file,"Finish runing iteration %d.\n",nIteration);
	fclose(file);
}

//debug purpose only
void CGibbsDP::OutputParam(double r,double* q,double** Theta)
{
	CString strParamfile;
	strParamfile=m_strOutput+"param.txt";
	
	FILE* file=fopen(strParamfile.m_pString,"a+");
	fprintf(file,"%.5lf\t",r);
	int k=0;
	for(k=0;k<m_nMotifNumber+1;k++)
		fprintf(file,"%.5lf\t",q[k]);

	for(k=0;k<m_nMotifNumber;k++)
	{
		for(int i=0;i<4*m_pnMotifWidth[k];i++)
		{
			fprintf(file,"%.3lf\t",Theta[k][i]);
		}
	}
	fprintf(file,"\n");
	fclose(file);

}

void CGibbsDP::ReadInputPWM(double** pdblTheta)				//read in the PWMs
{
	char pchrLine[1001];
	CString strLine;
	int nMaxChar=1000;
	FILE* file=fopen(m_strPWMFile.m_pString,"r");
	int k=-1;
	if(file)
	{
		while(fgets(pchrLine,nMaxChar,file))
		{
			if(pchrLine[0]=='>')
			{
				k++;
				strLine=pchrLine;
				strLine=strLine.Right(strLine.GetLength()-1);
				m_pnMotifWidth[k]=atoi(strLine.m_pString);
				if(pdblTheta[k]!=NULL) delete[] pdblTheta[k];
				pdblTheta[k]=new double[4*m_pnMotifWidth[k]];
				
				if(m_pnWeightMat[k]!=NULL) delete[] m_pnWeightMat[k];
				m_pnWeightMat[k]=new int[4*m_pnMotifWidth[k]];


				//read in the weight matrix
				for(int i=0;i<m_pnMotifWidth[k];i++)
				{
					fgets(pchrLine,nMaxChar,file);
					strLine=pchrLine;
					int nLength=strLine.GetLength();
					int nStart=0;
					int j=0;
					int nEnd=nStart;
					for(int b=0;b<4;b++)
					{
						for(j=nStart+1;j<nLength;j++)
						{
							if(strLine[j]=='\t' || strLine[j]=='\n')
							{
								nEnd=j;
								break;
							}
						}
						CString strTemp=strLine.Mid(nStart,nEnd-nStart);
						pdblTheta[k][4*i+b]=atof(strTemp.m_pString);
						nStart=nEnd;
						nEnd=nLength;
					}
				}
			}
		}
		fclose(file);
	}

	int nInitialPWMNumber=k+1;
	//normalize the weight matrices
	for(k=0;k<nInitialPWMNumber;k++)
	{
		for(int i=0;i<m_pnMotifWidth[k];i++)
		{
			double dblRowSum=0;
			int j=0;
			for(j=0;j<4;j++)
				dblRowSum+=pdblTheta[k][4*i+j];
			for(j=0;j<4;j++)
				pdblTheta[k][4*i+j]=(pdblTheta[k][4*i+j]+dblRowSum*0.0025)/(dblRowSum*1.01);
		}
	}
}


//debug purpose only
void CGibbsDP::SetInitialParam(double** pdblTheta)				//set initial parameters: debug usage
{
	double dblE2F[44]=
	{
		10,	 7,	 10,	 28, 
		14,	 12,	 15,	 14, 
		37,	 7,	 2,	 9, 
		6,	 2,	 0,	 47, 
		7,	 5,	 1,	 42, 
		6,	 2,	 2,	 45, 
		11,	 3,	 34,	 7, 
		3,	 50,	 0,	 2, 
		53,	 0,	 0,	 2 ,
		1,	 1,	 1,	 52,
		27,	 4,	 10,	 14
	};

	int i=0;
	for(i=0;i<11;i++)
	{
		double dblRowSum=0;
		int j=0;
		for(j=0;j<4;j++)
			dblRowSum+=dblE2F[i*4+j];
		for(j=0;j<4;j++)
		{
			pdblTheta[0][i*4+j]=(dblE2F[i*4+j]+dblRowSum*0.01)/(dblRowSum*1.04);
		}
	}

	double dblTFE[44]=
	{
		1,	 4,	 0,	 1,
		0,	 0,	 1,	 5,
		0,	 6,	 0,	 0, 
		1,	 0,	 0,	 5, 
		0,	 0,	 0,	 6, 
		0,	 0,	 0,	 6, 
		0,	 0,	 6,	 0, 
		1,	 0,	 0,	 5, 
		0,	 0,	 1,	 5, 
		4,	 1,	 0,	 1, 
		1,	 2,	 1,	 2 
	};

	for(i=0;i<11;i++)
	{
		double dblRowSum=0;
		int j=0;
		for(j=0;j<4;j++)
			dblRowSum+=dblTFE[i*4+j];
		for(j=0;j<4;j++)
		{
			pdblTheta[1][i*4+j]=(dblTFE[i*4+j]+dblRowSum*0.01)/(dblRowSum*1.04);
		}
	}
}


//sample the current sequence approximately
void CGibbsDP::ApproxSampleCurSeq(int n,double r_n,double* q_n,double** Theta_n,double* pdblLogPrPos,int* nMLPropIndex)	
{
	//calculate the log pr of each position under all the K+1 models
	int nLength=m_pstrSequence[n].GetLength();
	
	//calculate the sequence probability under the module model
	double* pdblLogPrModuleCurPos=new double[nLength];
	CalModuleLogPr(nLength-1,nLength,pdblLogPrPos,q_n,pdblLogPrModuleCurPos);

	//Calculate module probability and forward summation of Module
	double* pModuleLogPr=new double[nLength];
	double* pSumModuleLogPr=new double[nLength];
	int nModuleLength=m_nModuleLength;
	if(nModuleLength>nLength) nModuleLength=nLength;
	
	double dblSum=0;
	int i=0;
	for(i=0;i<nModuleLength-1;i++)
	{
		pModuleLogPr[i]=pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
		dblSum+=pModuleLogPr[i];//+log(1-r_n);
		pSumModuleLogPr[i]=dblSum;
	}
	for(i=nModuleLength-1;i<nLength;i++)
	{
		if(i>=nModuleLength)
			pModuleLogPr[i]=pdblLogPrModuleCurPos[i]-pdblLogPrModuleCurPos[i-nModuleLength];
		else
			pModuleLogPr[i]=pdblLogPrModuleCurPos[i];
		double dblLogP1=pSumModuleLogPr[i-1]+log(1-r_n)+pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
		double dblLogP2;
		if(i==nModuleLength-1)
			dblLogP2=log(r_n)+pModuleLogPr[i];
		else
			dblLogP2=pSumModuleLogPr[i-nModuleLength]+log(r_n)+pModuleLogPr[i];
		pSumModuleLogPr[i]=LogSum(dblLogP1,dblLogP2);
	}

	//record P(S|\psi,H_K)
	m_dblMAPLHK+=pSumModuleLogPr[nLength-1];

	//backward sampling
	ExcludeCurSeq(n);
	//1) sample modules
	SampleModule(pModuleLogPr,pSumModuleLogPr,r_n,nLength,pdblLogPrPos,n,nMLPropIndex,pdblLogPrModuleCurPos);

	delete[] pModuleLogPr;
	delete[] pSumModuleLogPr;	
	delete[] pdblLogPrModuleCurPos;
}

//backward sampling for the modules
void CGibbsDP::SampleModule(double* pModuleLogPr,double* pSumModuleLogPr,double r_n,
							int nLength,double* pdblLogPrPos,int n,int* nMLPropIndex,double* pdblLogPrModuleCurPos)
{
	//record the starting positions for the modules
	bool* pbModuleStart=new bool[nLength];
	int i=0;
	for(i=0;i<nLength;i++)
		pbModuleStart[i]=false;
	//record the module score
	m_pModule[n].dblLogModScore=0;
	m_pModule[n].dblLogMLCratio=0;

	double dblLogPrBg=0;
	double dblLogPrMd=0;
	i=nLength-1;
	int nModuleCount=0;
	double dblU;

	int nModuleLength=m_nModuleLength;
	if(nModuleLength>nLength) nModuleLength=nLength;


	while(i>=nModuleLength-1)
	{
		if(i==nModuleLength-1)
			dblLogPrMd=pModuleLogPr[i]+log(r_n)-pSumModuleLogPr[i];
		else
			dblLogPrMd=pSumModuleLogPr[i-nModuleLength]+pModuleLogPr[i]+log(r_n)-pSumModuleLogPr[i];

		bool bModule=false;
		if(dblLogPrMd>-200)
		{
			dblU=double(rand())/(RAND_MAX);
			if(dblU<exp(dblLogPrMd))
			{
				bModule=true;
			}
		}	

		if(m_bMotifMode) bModule=true;

		if(bModule)
		{
			pbModuleStart[i-nModuleLength+1]=true;
			nModuleCount++;
			CString strModule=m_pstrSequence[n].Mid(i-nModuleLength+1,nModuleLength);
			double dblLogBgPr=CalLogBgPr(strModule);
			m_pModule[n].dblLogModScore+=(pModuleLogPr[i]-dblLogBgPr);

			//calcuate metropolis ratio for module length change 
			int nLeft;
			int nRight;
			if(*nMLPropIndex>=0)
			{
				if(*nMLPropIndex==0)	nLeft=i-nModuleLength-m_nDeltL+1;
				else if(*nMLPropIndex==1) nLeft=i+1;
				else if(*nMLPropIndex==2) nLeft=i-nModuleLength+1;
				else if(*nMLPropIndex==3) nLeft=i-m_nDeltL+1;
				nRight=nLeft+m_nDeltL-1;
				if(nLeft<0 || nRight>=nLength) *nMLPropIndex=-1;
				else
				{
					if(nLeft>=1)
						m_pModule[n].dblLogMLCratio+=(pdblLogPrModuleCurPos[nRight]-pdblLogPrModuleCurPos[nLeft-1]);
					else if(nLeft==0)
						m_pModule[n].dblLogMLCratio+=pdblLogPrModuleCurPos[nRight];
					CString strModChg=m_pstrSequence[n].Mid(nLeft,m_nDeltL);
					m_pModule[n].dblLogMLCratio-=CalLogBgPr(strModChg);
				}
			}

			i=i-nModuleLength;
		}
		else
			i--;
	}

	//record the modules
	m_pModule[n].nCount=nModuleCount;
	if(nModuleCount>0)
	{
		m_pModule[n].pnStart=new int[nModuleCount];
		int nCount=0;
		for(i=nLength-1;i>=0;i--)
		{
			if(pbModuleStart[i])
			{
				m_pModule[n].pnStart[nCount]=i;
				nCount++;
			}
		}
	}
	delete[] pbModuleStart;
}

void CGibbsDP::UpdateModuleLength(int nMLPropIndex)		//sampling the module length given current modules
{
	double dblLogMLCLike=0;
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		dblLogMLCLike+=m_pModule[n].dblLogMLCratio;
	}
	int nNewModuleLength;
	double dblLogPriorRatio=0;
	double dblLogPosterRatio=0;
	if(nMLPropIndex>=2)	//shrink
	{
		nNewModuleLength=m_nModuleLength-m_nDeltL;
		dblLogPriorRatio=log(15.0/14);
		dblLogPosterRatio=dblLogPriorRatio-dblLogMLCLike;
	}
	else if(nMLPropIndex<2) //extend
	{
		nNewModuleLength=m_nModuleLength+m_nDeltL;
		dblLogPriorRatio=log(14/15.0);
		dblLogPosterRatio=dblLogPriorRatio+dblLogMLCLike;
	}

	//Metropolis update
	bool bAccept=false;
	if(dblLogPosterRatio>0) bAccept=true;
	else if(dblLogPosterRatio<-200) bAccept=false;
	else
	{
		double dblU=double(rand())/RAND_MAX;
		if(dblU<exp(dblLogPosterRatio))
			bAccept=true;
	}

	if(bAccept)
	{
		m_nModuleLength=nNewModuleLength;
		if(nMLPropIndex%2==0) //left side of module:change module start
		{
			int nDelt=m_nDeltL;
			if(nMLPropIndex==0) nDelt=-m_nDeltL;
			for(int n=0;n<m_nSequenceNumber;n++)
			{
				int nCount=m_pModule[n].nCount;
				for(int m=0;m<nCount;m++)
					m_pModule[n].pnStart[m]+=nDelt;
			}

		}
	}
}

void CGibbsDP::ExcludeCurSeq(int n)			//exclude current sequence
{
	//module:
	m_pModule[n].nCount=0;
	if(m_pModule[n].pnStart!=NULL)
	{
		delete[] m_pModule[n].pnStart;
		m_pModule[n].pnStart=NULL;
	}

	//sites:
	for(int k=0;k<m_nMotifNumber;k++)
	{
		SITEINFO* pCurSite=m_pSiteHead[k][n]->pNextSite;
		SITEINFO* pDel;
		while(pCurSite)
		{
			m_pnSiteCount[k]--;

			CString strSite=pCurSite->strSite;
			//change the marginal weight matrix
			for(int i=0;i<m_pnMotifWidth[k];i++)
			{
				switch(strSite[i])
				{
				case 'A':
				case 'a':
					m_pnWeightMat[k][i*4]--;
					break;
				case 'C':
				case 'c':
					m_pnWeightMat[k][i*4+1]--;
					break;
				case 'G':
				case 'g':
					m_pnWeightMat[k][i*4+2]--;
					break;
				case 'T':
				case 't':
					m_pnWeightMat[k][i*4+3]--;
					break;
				}
			}
			pDel=pCurSite;
			pCurSite=pCurSite->pNextSite;
			delete pDel;
		}
		m_pSiteHead[k][n]->pNextSite=NULL;
	}
}

//forward summation of the marginal probability of being a module
double CGibbsDP::CalModuleLogPr(int nEnd,int nLength,double* pdblLogPrPos,double* q_n,double* pdblPr)					//calculate the log pr of module
{
	int nMinWidth=m_pnMotifWidth[Minimum(m_pnMotifWidth,m_nMotifNumber)];

	int nStart=nEnd-nLength+1;
	pdblPr[0]=pdblLogPrPos[nStart*(2*m_nMotifNumber+1)+2*m_nMotifNumber]
		+log(q_n[m_nMotifNumber]);
	double dblLogPrM;
	for(int i=nStart+1;i<=nEnd;i++)
	{
		pdblPr[i-nStart]=pdblPr[i-nStart-1]+pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber]+log(q_n[m_nMotifNumber]);
//		if(i-nStart+1>=nMinWidth)
//			pdblPr[i-nStart]+=log(q_n[m_nMotifNumber]);
		if(!m_bReverse)	//only forward chain
		{
			for(int k=0;k<m_nMotifNumber;k++)
			{
				if(i-nStart+1>=m_pnMotifWidth[k])
				{
					dblLogPrM=pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k]+log(q_n[k]);
				}
				if(i-nStart+1>m_pnMotifWidth[k])
				{
					dblLogPrM+=pdblPr[i-nStart-m_pnMotifWidth[k]];						
				}
				if(i-nStart+1>=m_pnMotifWidth[k])
					pdblPr[i-nStart]=LogSum(pdblPr[i-nStart],dblLogPrM);
			}
		}
		else	//both strand
		{
			for(int k=0;k<m_nMotifNumber;k++)
			{
				if(i-nStart+1>=m_pnMotifWidth[k])
				{
					dblLogPrM=
						LogSum(pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k],pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k+1]);
					dblLogPrM+=log(q_n[k])+log(0.5);
				}
				if(i-nStart+1>m_pnMotifWidth[k])
				{
					dblLogPrM+=pdblPr[i-nStart-m_pnMotifWidth[k]];
				}
				if(i-nStart+1>=m_pnMotifWidth[k])
					pdblPr[i-nStart]=LogSum(pdblPr[i-nStart],dblLogPrM);
			}
		}
	}
	//get the minimum width of all the motifs
	double dblLogPrModule=pdblPr[nLength-1];
	return dblLogPrModule;
}

void CGibbsDP::SampleSites(double* q_n,double* pdblLogPrPos,int n,int nEnd, int nModuleLength)
{
	int nMinWidth=m_pnMotifWidth[Minimum(m_pnMotifWidth,m_nMotifNumber)];
	
	//forward summation:
	double* pdblPr=new double[nModuleLength];
	int nStart=nEnd-nModuleLength+1;
	pdblPr[0]=pdblLogPrPos[nStart*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
	//	+log(q_n[m_nMotifNumber]);
	double dblLogPrM;
	int i=0;
	for(i=nStart+1;i<=nEnd;i++)
	{
		pdblPr[i-nStart]=pdblPr[i-nStart-1]+pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
		if(i-nStart+1>=nMinWidth)
			pdblPr[i-nStart]+=log(q_n[m_nMotifNumber]);
		if(!m_bReverse)	//only forward chain
		{
			for(int k=0;k<m_nMotifNumber;k++)
			{
				if(i-nStart+1>=m_pnMotifWidth[k])
				{
					dblLogPrM=pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k]+log(q_n[k]);
				}
				if(i-nStart+1>m_pnMotifWidth[k])
				{
					dblLogPrM+=pdblPr[i-nStart-m_pnMotifWidth[k]];						
				}
				if(i-nStart+1>=m_pnMotifWidth[k])
					pdblPr[i-nStart]=LogSum(pdblPr[i-nStart],dblLogPrM);
			}
		}
		else	//both strand
		{
			for(int k=0;k<m_nMotifNumber;k++)
			{
				if(i-nStart+1>=m_pnMotifWidth[k])
				{
					dblLogPrM=
						LogSum(pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k],pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*k+1]);
					dblLogPrM+=log(q_n[k])+log(0.5);
				}
				if(i-nStart+1>m_pnMotifWidth[k])
				{
					dblLogPrM+=pdblPr[i-nStart-m_pnMotifWidth[k]];
				}
				if(i-nStart+1>=m_pnMotifWidth[k])
					pdblPr[i-nStart]=LogSum(pdblPr[i-nStart],dblLogPrM);
			}
		}
	}

	//backward sampling
	i=nModuleLength-1;
	double* pdblMotifPrCdf;
	double pdblMotifPr;

	if(!m_bReverse)		//only forward chain is sampling
	{
		pdblMotifPrCdf=new double[m_nMotifNumber];
		while(i>=nMinWidth-1)
		{
			double dblSum=0;
			int k=0;
			for(k=0;k<m_nMotifNumber;k++)
			{
				if(i>=m_pnMotifWidth[k])
					pdblMotifPr=pdblPr[i-m_pnMotifWidth[k]]
					+pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+2*k]+log(q_n[k]);
				else if(i==m_pnMotifWidth[k]-1)
					pdblMotifPr=pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+2*k]+log(q_n[k]);
				else
					pdblMotifPr=-2000+pdblPr[i];
				pdblMotifPr-=pdblPr[i];

				if(pdblMotifPr<-200)
					pdblMotifPr=0;
				else
					pdblMotifPr=exp(pdblMotifPr);
				dblSum+=pdblMotifPr;
				pdblMotifPrCdf[k]=dblSum;
			}
			double dblBg=pdblPr[i-1]
					+pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+2*m_nMotifNumber]+log(q_n[k])-pdblPr[i];
			dblBg=exp(dblBg);

			double dblU=double(rand())/RAND_MAX;
			int nMotif=-1;
			for(k=0;k<m_nMotifNumber;k++)
			{
				if(dblU<pdblMotifPrCdf[k])
				{
					nMotif=k;
					break;
				}
			}

			if(nMotif>=0)
			{
				SITEINFO* pNewSite=new SITEINFO;
				pNewSite->bForward=true;
				pNewSite->nPos=nStart+i-m_pnMotifWidth[nMotif]+1;
				pNewSite->strSite=m_pstrSequence[n].Mid(pNewSite->nPos,m_pnMotifWidth[nMotif]);
				pNewSite->pNextSite=NULL;
				AddSite(m_pSiteHead[nMotif][n],pNewSite);
				i=i-m_pnMotifWidth[nMotif];
			}
			else
				i--;
		}
	}
	else	//sampling both chain
	{
		pdblMotifPrCdf=new double[m_nMotifNumber*2];
		while(i>=nMinWidth-1)
		{
			double dblSum=0;
			int k=0;
			for(k=0;k<2*m_nMotifNumber;k++)
			{
				if(i>=m_pnMotifWidth[k/2])
					pdblMotifPr=pdblPr[i-m_pnMotifWidth[k/2]]
					+pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+k]+log(q_n[k/2])+log(0.5);
				else if(i==m_pnMotifWidth[k/2]-1)
					pdblMotifPr=pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+k]+log(q_n[k/2])+log(0.5);
				else
					pdblMotifPr=-2000+pdblPr[i];
				pdblMotifPr-=pdblPr[i];

				if(pdblMotifPr<-200)
					pdblMotifPr=0;
				else
					pdblMotifPr=exp(pdblMotifPr);
				dblSum+=pdblMotifPr;
				pdblMotifPrCdf[k]=dblSum;
			}
			double dblBg=pdblPr[i-1]
					+pdblLogPrPos[(nStart+i)*(2*m_nMotifNumber+1)+2*m_nMotifNumber]+log(q_n[m_nMotifNumber])-pdblPr[i];

			double dblU=double(rand())/RAND_MAX;
			int nMotif=-1;
			for(k=0;k<2*m_nMotifNumber;k++)
			{
				if(dblU<pdblMotifPrCdf[k])
				{
					nMotif=k;
					break;
				}
			}

			if(nMotif>=0)
			{
				SITEINFO* pNewSite=new SITEINFO;
				if(nMotif%2==0)
					pNewSite->bForward=true;
				else
					pNewSite->bForward=false;
				pNewSite->nPos=nStart+i-m_pnMotifWidth[nMotif/2]+1;
				if(pNewSite->bForward)
					pNewSite->strSite=m_pstrSequence[n].Mid(pNewSite->nPos,m_pnMotifWidth[nMotif/2]);
				else
					pNewSite->strSite=Reverse(m_pstrSequence[n].Mid(pNewSite->nPos,m_pnMotifWidth[nMotif/2]));
				pNewSite->pNextSite=NULL;
				AddSite(m_pSiteHead[nMotif/2][n],pNewSite);
				i=i-m_pnMotifWidth[nMotif/2];
			}
			else
				i--;
		}
	}

	delete[] pdblMotifPrCdf;
	delete[] pdblPr;
}

void CGibbsDP::ReadInputSeq()
{
	//read in sequence file
	int nMaxChar=10000;
	FILE* InputFile;
	InputFile=fopen(m_strInputSeq.m_pString,"r");
	if(InputFile)
	{
		int i=0;
		char pcharLine[10001];
		while(fgets(pcharLine,nMaxChar,InputFile))
		{
			if(pcharLine[0]=='>') i++;
		}
		m_nSequenceNumber=i;		//number of sequences
		fclose(InputFile);
	}
	else
	{
		printf("Input sequence file error!\n");
		exit(0);
	}
	if(m_nSequenceNumber<=0)
	{
		printf("Input sequence file error!\n");
		exit(0);
	}
	m_pstrSequence=new CString[m_nSequenceNumber];
	m_pstrSeqName=new CString[m_nSequenceNumber];

	//read in sequences and calculate the total lenght L (-|S|*l)
	m_nTotalLength=0;
	InputFile=fopen(m_strInputSeq.m_pString,"r");
	CString strLine;
	if(InputFile)
	{
		int i=-1;
		char pcharLine[10001];
		while(fgets(pcharLine,nMaxChar,InputFile))
		{
			if(pcharLine[0]=='>')
			{
				i++;
				m_pstrSeqName[i]=pcharLine;
				m_pstrSeqName[i]=m_pstrSeqName[i].Left(m_pstrSeqName[i].GetLength()-1);
			}
			else
			{
				strLine=pcharLine;
				//change 'a'->'A'
				int nLength=strLine.GetLength();
				for(int j=0;j<nLength;j++)
				{
					if(strLine[j]>='a' && strLine[j] <='z')
					{
						char ch=strLine[j];
						ch+='A'-'a';
						strLine.SetAt(j,ch);
					}
					else if(strLine[j]<'A' || strLine[j]>'Z')
					{
						nLength=j;
						break;
					}
				}
				strLine=strLine.Left(nLength);
				m_pstrSequence[i]=m_pstrSequence[i]+strLine;
				m_nTotalLength+=nLength;
			}
		}
		fclose(InputFile);
	}
}

void CGibbsDP::BuildBackground()				//build background by the input seq.
{
	int i=0;
	for(i=0;i<20;i++)
		m_dblBackGround[i]=0;
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		int nLength=m_pstrSequence[n].GetLength();
		CString strDouble;
		for(i=0;i<nLength;i++)
		{
			//calculate the frequenc of A C G T
			switch(m_pstrSequence[n][i])
			{
			case 'A':
				m_dblBackGround[0]++;
				break;
			case 'T':
				m_dblBackGround[3]++;
				break;
			case 'C':
				m_dblBackGround[1]++;
				break;
			case 'G':
				m_dblBackGround[2]++;
				break;
			}
			//calculate the frequence of AA, AC, AG, AT...
			if(i<nLength-1)
			{
				strDouble=m_pstrSequence[n].Mid(i,2);
				if(strDouble.Find("N")>-1) continue;
				int nCode=WordCode(strDouble);
				m_dblBackGround[nCode+4]++;
			}
		}
	}
	//Transition Pr of AA AC ...
	for(int j=0;j<4;j++)
		for(i=0;i<4;i++)
			m_dblBackGround[(j+1)*4+i]/=m_dblBackGround[j];
	double dblTotal=0;
	//get the Pr of A C G T
	for(i=0;i<4;i++)
		dblTotal+=m_dblBackGround[i];
	for(i=0;i<4;i++)
		m_dblBackGround[i]/=dblTotal;
}

//calculate the log pr of each position under all the K+1 models
void CGibbsDP::CalLogPrPos(double** pdblLogPrPos,int* pnLength,double** Theta_n)		//calculate the log probability for each position 
{
	CString strWindow;
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		for(int i=0;i<pnLength[n];i++)
		{
			//calculate the motif pr [i-w+1,i]
			for(int k=0;k<m_nMotifNumber;k++)
			{
				if(i<m_pnMotifWidth[k]-1)
				{
					pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*k]=-2000;
					pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*k+1]=-2000;				
				}
				else
				{
					strWindow=m_pstrSequence[n].Mid(i-m_pnMotifWidth[k]+1,m_pnMotifWidth[k]);
					//check if strWindow contains "N" or "n"
					if(strWindow.Find("N") >-1 || strWindow.Find("n") >-1)
					{
						pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*k]=-2000;
						pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*k+1]=-2000;
						continue;
					}

					pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*k]=CalLogMotifPr(strWindow,Theta_n[k]);
					if(m_bReverse)
						pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*k+1]=CalLogMotifPr(Reverse(strWindow),Theta_n[k]);
					else
						pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*k+1]=-2000;
				}
			}
		}
	}
}

//calculate log background probabilities
void CGibbsDP::CalLogBgPrPos(double** pdblLogPrPos,int* pnLength)
{
	if(!m_bInputBackground) //calculate by 1st order MC
	{
		CString strWindow;
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			for(int i=0;i<pnLength[n];i++)
			{
				//calculate the background transition
				if(i==0)
				{
					//if the current position is 'N' or 'n':
					if(m_pstrSequence[n][0]=='N' || m_pstrSequence[n][0]=='n')
						pdblLogPrPos[n][2*m_nMotifNumber]=-1.386; //log(1/4);
					else
					{
						CString strtemp=m_pstrSequence[n][0];
						pdblLogPrPos[n][2*m_nMotifNumber]=log(m_dblBackGround[WordCode(strtemp)]);
					}
				}
				else
				{
					strWindow=m_pstrSequence[n].Mid(i-1,2);

					if(strWindow.Find("N") >-1 || strWindow.Find("n") >-1)
					{
						pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*m_nMotifNumber]=-1.386;
					}
					else
					{
						int nFirst=WordCode(strWindow[0]);
						int nSecond=WordCode(strWindow[1]);
						pdblLogPrPos[n][i*(2*m_nMotifNumber+1)+2*m_nMotifNumber]=log(m_dblBackGround[nFirst*4+nSecond+4]);
					}
				}
			}
		}
	}
	else	//read input background probabilities
	{
		FILE* file=fopen(m_strInputBgfile.m_pString,"r");
		if(file)
		{
			char pchrTemp[1000];
			double pdblData[7];
			CString strLine;
			while(fgets(pchrTemp,1000,file))
			{
				strLine=pchrTemp;
				if(strLine.GetLength()>10)
				{
					ReadData(strLine,0,7,pdblData);
					int n=int(pdblData[0]+0.1)-1;
					int i=int(pdblData[6]+0.1)-1;
					int nchar=WordCode(m_pstrSequence[n][i]);
					if(nchar==0)
						pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*m_nMotifNumber]=log(pdblData[2]);
					else if(nchar==3)
						pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*m_nMotifNumber]=log(pdblData[3]);
					else if(nchar==1)
						pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*m_nMotifNumber]=log(pdblData[4]);
					else if(nchar==2)
						pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*m_nMotifNumber]=log(pdblData[5]);
					else
						pdblLogPrPos[n][i*(m_nMotifNumber*2+1)+2*m_nMotifNumber]=-1.386;
				}
			}
			fclose(file);
		}
	}
}

void CGibbsDP::RandPickSite()				//pick one site from each sequence randomly
{
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		int nSeqLength=m_pstrSequence[n].GetLength();
		int nModuleLength=m_nModuleLength;
		if(nModuleLength>nSeqLength) nModuleLength=nSeqLength;

		int nPieceLength=(nModuleLength-Sum(m_pnMotifWidth,m_nMotifNumber))/m_nMotifNumber;

		//random pick the starting pos for the module
		m_pModule[n].nCount=1;
		m_pModule[n].pnStart=new int;
		*(m_pModule[n].pnStart)=rand()%(nSeqLength-nModuleLength+1);

		//randomly initialize the sites
		int nSumMotifWidth=0;
		for(int k=0;k<m_nMotifNumber;k++)
		{
			int nPos=rand()%(1+nPieceLength);
			SITEINFO* pNewSite=new SITEINFO;
			pNewSite->bForward=true;
			pNewSite->nPos=*(m_pModule[n].pnStart)+k*nPieceLength+nSumMotifWidth+nPos;
			pNewSite->pNextSite=NULL;
			pNewSite->strSite=m_pstrSequence[n].Mid(pNewSite->nPos,m_pnMotifWidth[k]);
			AddSite(m_pSiteHead[k][n],pNewSite);

			nSumMotifWidth+=m_pnMotifWidth[k];
		}
	}

	//calculate the baysian pp score:
	m_dblOptLogJPP=CalLogPostPrSample();
	CopySample(m_pSiteHead,m_pOptSiteHead);
}

double CGibbsDP::CalLogPostPrSample()
{
	int k=0;
	for(k=0;k<m_nMotifNumber;k++)
	{
		m_pnSiteCount[k]=CalWeightMat(m_pSiteHead[k],m_pnWeightMat[k],m_pnMotifWidth[k]);
	}
	//obtain the module count
	int nModuleCount=CountModule(m_pModule);
	//calculate the background log pr for all the sites
	double dblLogBgPrSite=0;
	for(k=0;k<m_nMotifNumber;k++)
	{
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			SITEINFO* pCurSite=m_pSiteHead[k][n]->pNextSite;
			while(pCurSite)
			{
				dblLogBgPrSite+=CalLogBgPr(pCurSite->strSite);
				pCurSite=pCurSite->pNextSite;
			}
		}
	}

	//calculate the baysian pp score:
	double dblLogPP=
		CalBayesianScore(nModuleCount,m_pnSiteCount,m_pnWeightMat,m_pnMotifWidth,dblLogBgPrSite);

	//add up the module score
	double dblLogModScore=0;
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		dblLogModScore+=m_pModule[n].dblLogModScore;
	}

	dblLogPP+=dblLogModScore;
	return dblLogPP;
}

double CGibbsDP::CalBayesianScore(int nModuleCount,int* pnSiteCount,int** pnWeightMat,int* pnMotifWidth,double dblLogBgPr)
{
	double dblLogScore=0;

	//module number:
	dblLogScore+=logGamma(nModuleCount+m_pnPseudoModuleCounts[0]);
	dblLogScore+=logGamma(m_nTotalLength-nModuleCount*m_nModuleLength+m_pnPseudoModuleCounts[1]);
	dblLogScore-=logGamma(m_nTotalLength-nModuleCount*(m_nModuleLength-1)+m_pnPseudoModuleCounts[0]+m_pnPseudoModuleCounts[1]);

	//site numbers:
	int nMBCount=nModuleCount*m_nModuleLength;
	int k=0;
	for(k=0;k<m_nMotifNumber;k++)
	{
		nMBCount-=pnSiteCount[k];//*pnMotifWidth[k];
	}

	int nTotalCount=nMBCount+m_pnPseudoSiteCounts[m_nMotifNumber];
	dblLogScore+=logGamma(nTotalCount);
	for(k=0;k<m_nMotifNumber;k++)
	{
		dblLogScore+=logGamma(pnSiteCount[k]+m_pnPseudoSiteCounts[k]);
		nTotalCount+=(pnSiteCount[k]+m_pnPseudoSiteCounts[k]);
	}
	dblLogScore-=logGamma(nTotalCount);

	//weight matrices:
	int nSumPseudo=0;
	for(int j=0;j<4;j++)
		nSumPseudo+=m_pnPseudoBpCounts[j];

	for(k=0;k<m_nMotifNumber;k++)
	{
		for(int i=0;i<pnMotifWidth[k];i++)
		{
			for(int j=0;j<4;j++)
			{
				dblLogScore+=logGamma(pnWeightMat[k][4*i+j]+m_pnPseudoBpCounts[j]);
			}
		}
		dblLogScore-=pnMotifWidth[k]*logGamma(pnSiteCount[k]+nSumPseudo);
	}

	//background
	dblLogScore-=dblLogBgPr;

	//prior for w
	for(k=0;k<m_nMotifNumber;k++)
	{
		dblLogScore+=pnMotifWidth[k]*log(15)-logGamma(pnMotifWidth[k]+1);
	}

	return dblLogScore;
}

void CGibbsDP::CopySample(SITEINFO*** pSourceSites,SITEINFO*** pDestSites)		//copy the site info
{
	for(int k=0;k<m_nMotifNumber;k++)
	{
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			//delete the original pDestSite
			SITEINFO* pCurSite=pDestSites[k][n]->pNextSite;
			while(pCurSite)
			{
				SITEINFO* pDel=pCurSite;
				pCurSite=pCurSite->pNextSite;
				delete pDel;
			}
			pDestSites[k][n]->pNextSite=NULL;

			//copy the new sites
			pCurSite=pSourceSites[k][n]->pNextSite;
			SITEINFO* pDest=pDestSites[k][n];
			while(pCurSite)
			{
				SITEINFO* pNewSite=new SITEINFO;
				pNewSite->bForward=pCurSite->bForward;
				pNewSite->nPos=pCurSite->nPos;
				pNewSite->nMA=pCurSite->nMA;
				pNewSite->strSite=pCurSite->strSite;
				pNewSite->pNextSite=NULL;

				pDest->pNextSite=pNewSite;
				
				pDest=pNewSite;
				pCurSite=pCurSite->pNextSite;
			}
		}
	}
}

void CGibbsDP::CopySample(SITEINFO** pSourceSites,SITEINFO** pDestSites)		//copy the site info
{
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		//delete the original pDestSite
		SITEINFO* pCurSite=pDestSites[n]->pNextSite;
		while(pCurSite)
		{
			SITEINFO* pDel=pCurSite;
			pCurSite=pCurSite->pNextSite;
			delete pDel;
		}
		pDestSites[n]->pNextSite=NULL;

		//copy the new sites
		pCurSite=pSourceSites[n]->pNextSite;
		SITEINFO* pDest=pDestSites[n];
		while(pCurSite)
		{
			SITEINFO* pNewSite=new SITEINFO;
			pNewSite->bForward=pCurSite->bForward;
			pNewSite->nPos=pCurSite->nPos;
			pNewSite->strSite=pCurSite->strSite;
			pNewSite->nMA=pCurSite->nMA;
			pNewSite->pNextSite=NULL;

			pDest->pNextSite=pNewSite;
			
			pDest=pNewSite;
			pCurSite=pCurSite->pNextSite;
		}
	}
}

void CGibbsDP::AddSite(SITEINFO* pSiteHead,SITEINFO* pNewSite)
{
	pNewSite->pNextSite=pSiteHead->pNextSite;
	pSiteHead->pNextSite=pNewSite;
}

double CGibbsDP::CalLogBgPr(CString strSite)			//Calculate log background probabilities
{
	//first letter
	double dblPrLogBackground=log(m_dblBackGround[WordCode(strSite[0])]);
	int nMotifWidth=strSite.GetLength();
	CString strDouble;
	for(int i=0;i<nMotifWidth-1;i++)
	{
		strDouble=strSite.Mid(i,2);
		dblPrLogBackground+=log(m_dblBackGround[4+WordCode(strDouble)]);
	}
	return dblPrLogBackground;
}

//calculate log Motif probability
double CGibbsDP::CalLogMotifPr(CString strWindow,double* Theta)
{
	double dblLogPrMotif=0;
	int nMotifWidth=strWindow.GetLength();
	int nBase;
	for(int i=0;i<nMotifWidth;i++)
	{
		nBase=WordCode(strWindow[i]);
		dblLogPrMotif+=log(Theta[i*4+nBase]);
	}
	return dblLogPrMotif;
}

double CGibbsDP::CalFixSiteScore(int* pnWeightMat,int nMotifWidth,double dblLogBgPr)
{
	double dblLogScore=-dblLogBgPr;
	for(int i=0;i<nMotifWidth;i++)
	{
		for(int j=0;j<4;j++)
		{
			dblLogScore+=logGamma(pnWeightMat[4*i+j]+m_pnPseudoBpCounts[j]);
		}
	}
	return dblLogScore;
}

int CGibbsDP::CountModule(MODULEINFO* pModule)		
{
	int nCount=0;
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		nCount+=pModule[n].nCount;
	}
	return nCount;
}

//calculate weight matrix for one motif
int CGibbsDP::CalWeightMat(SITEINFO** pSiteHead,int* pnWeightMat,int nWidth)
{
	for(int i=0;i<nWidth;i++)
	{
		for(int j=0;j<4;j++)
			pnWeightMat[i*4+j]=0;
	}

	SITEINFO* pCurSite;
	int nSiteCount=0;
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		pCurSite=pSiteHead[n]->pNextSite;
		while(pCurSite)
		{
			for(int i=0;i<nWidth;i++)
			{
				CString strSite=pCurSite->strSite;
				switch(strSite[i])
				{
				case 'A':
				case 'a':
					pnWeightMat[i*4]++;
					break;
				case 'C':
				case 'c':
					pnWeightMat[i*4+1]++;
					break;
				case 'G':
				case 'g':
					pnWeightMat[i*4+2]++;
					break;
				case 'T':
				case 't':
					pnWeightMat[i*4+3]++;
					break;
				}
			}			
			pCurSite=pCurSite->pNextSite;
			nSiteCount++;
		}
	}//for
	return nSiteCount;
}

void CGibbsDP::UpdateWidth(int k)				//sampling the motif width given current sites: k the motif index
{
	bool pbBoundary[2];
	CheckBoundary(m_pSiteHead[k],&(pbBoundary[0]));

	//four possibility: (L,+1), (L,-1), (R,+1), (R,-1)
	double dblU=double(rand())/(RAND_MAX+1);
	bool bLeft=true;
	if(dblU>0.5) bLeft=false;

	int nInc=-1;
	dblU=double(rand())/(RAND_MAX+1);
	if(dblU>0.5) nInc=1;

	if(bLeft && nInc==1 && pbBoundary[0])		//left increase width
		return;
	if((!bLeft) && nInc==1 && pbBoundary[1])		//rigt increase width
		return;

	//propose new width
	int nNewWidth=m_pnMotifWidth[k]+nInc;
	if(nNewWidth<m_nMinWidth ||nNewWidth>m_nMaxWidth)
		return;
	//shift of the initial positions
	int nFShift=0;	//forward site
	if(bLeft)		
		nFShift=-nInc;

	int nBShift=0;	//backward site
	if(!bLeft)		
		nBShift=-nInc;

	//set up new samples and calculate the background log pr
	double dblLogBgPrNewSample=0;
	double dblLogBgPrCurSample=0;
	SITEINFO** pNewWidthSample=new SITEINFO*[m_nSequenceNumber];
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		pNewWidthSample[n]=new SITEINFO;
		pNewWidthSample[n]->pNextSite=NULL;
		
		SITEINFO* pCurSite=m_pSiteHead[k][n]->pNextSite;
		SITEINFO* pNewWidthCurSite=pNewWidthSample[n];
		while(pCurSite)
		{
			SITEINFO* pNewSite=new SITEINFO;
			pNewSite->bForward=pCurSite->bForward;
			pNewSite->pNextSite=NULL;
			if(pNewSite->bForward)
			{
				pNewSite->nPos=pCurSite->nPos+nFShift;
				pNewSite->strSite=m_pstrSequence[n].Mid(pNewSite->nPos,nNewWidth);
			}
			else
			{
				pNewSite->nPos=pCurSite->nPos+nBShift;
				pNewSite->strSite=Reverse(m_pstrSequence[n].Mid(pNewSite->nPos,nNewWidth));
			}
			dblLogBgPrNewSample+=CalLogBgPr(pNewSite->strSite);
			dblLogBgPrCurSample+=CalLogBgPr(pCurSite->strSite);
			pNewWidthCurSite->pNextSite=pNewSite;
			pNewWidthCurSite=pNewSite;
			pCurSite=pCurSite->pNextSite;
		}
	}

	int* pnNewWeightMat=new int[nNewWidth*4];
	int pnSiteCount=CalWeightMat(pNewWidthSample,pnNewWeightMat,nNewWidth);

	//site numbers:
	int nModuleCount=CountModule(m_pModule);
	int nMBCount=nModuleCount*(m_nModuleLength-15);
	int i=0;
	for(i=0;i<m_nMotifNumber;i++)
	{
		nMBCount-=m_pnSiteCount[i];//*m_pnMotifWidth[i];
	}
	int nTotalCount=nMBCount+m_pnPseudoSiteCounts[m_nMotifNumber];
	for(i=0;i<m_nMotifNumber;i++)
	{
		nTotalCount+=(m_pnSiteCount[i]+m_pnPseudoSiteCounts[i]);
	}

	int nNewMBCount=nMBCount;//-nInc*m_pnSiteCount[k];
	int nNewTotalCount=nTotalCount;//-nInc*m_pnSiteCount[k];

	//calculate P(w|I,M,S)
	double dblCurLog=
		CalLogPCondPW(nMBCount,nTotalCount,m_pnMotifWidth[k],m_pnWeightMat[k],dblLogBgPrCurSample);
	double dblNewLog=
		CalLogPCondPW(nNewMBCount,nNewTotalCount,nNewWidth,pnNewWeightMat,dblLogBgPrNewSample);

	//MH
	bool bAccept=MH(dblCurLog,dblNewLog,0);
	if(bAccept)
	{
		m_pnMotifWidth[k]=nNewWidth;
		delete[] m_pnWeightMat[k];
		m_pnWeightMat[k]=pnNewWeightMat;
		DelSiteHead(m_pSiteHead[k]);
		m_pSiteHead[k]=pNewWidthSample;
	}
	else
	{
		delete[] pnNewWeightMat;
		DelSiteHead(pNewWidthSample);
	}
}

void CGibbsDP::PhaseShift(int k)			//phase shift proposal of the current chain
{
	bool pbBoundary[2];
	CheckBoundary(m_pSiteHead[k],&(pbBoundary[0]));

	//four possibility: (L,+1), (L,-1), (R,+1), (R,-1)
	double dblU=double(rand())/(RAND_MAX+1);
	bool bLeft=true;
	if(dblU>0.5) bLeft=false;

	if(bLeft && pbBoundary[0])		//left increase width
		return;
	if((!bLeft) && pbBoundary[1])		//rigt increase width
		return;

	//shift of the initial positions
	int nFShift=0;	//forward site
	int nBShift=0;	//backward site
	if(bLeft)	
	{
		nFShift=-1;
		nBShift=1;
	}
	else
	{
		nFShift=1;
		nBShift=-1;
	}


	//set up new samples and calculate the background log pr
	double dblLogBgPrNewSample=0;
	double dblLogBgPrCurSample=0;
	SITEINFO** pShiftSample=new SITEINFO*[m_nSequenceNumber];
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		pShiftSample[n]=new SITEINFO;
		pShiftSample[n]->pNextSite=NULL;
		
		SITEINFO* pCurSite=m_pSiteHead[k][n]->pNextSite;
		SITEINFO* pShiftCurSite=pShiftSample[n];
		while(pCurSite)
		{
			SITEINFO* pNewSite=new SITEINFO;
			pNewSite->bForward=pCurSite->bForward;
			pNewSite->pNextSite=NULL;
			if(pNewSite->bForward)
			{
				pNewSite->nPos=pCurSite->nPos+nFShift;
				pNewSite->strSite=m_pstrSequence[n].Mid(pNewSite->nPos,m_pnMotifWidth[k]);
			}
			else
			{
				pNewSite->nPos=pCurSite->nPos+nBShift;
				pNewSite->strSite=Reverse(m_pstrSequence[n].Mid(pNewSite->nPos,m_pnMotifWidth[k]));
			}
			dblLogBgPrNewSample+=CalLogBgPr(pNewSite->strSite);
			dblLogBgPrCurSample+=CalLogBgPr(pCurSite->strSite);
			pShiftCurSite->pNextSite=pNewSite;
			pShiftCurSite=pNewSite;
			pCurSite=pCurSite->pNextSite;
		}
	}

	int* pnNewWeightMat=new int[m_pnMotifWidth[k]*4];
	int pnSiteCount=CalWeightMat(pShiftSample,pnNewWeightMat,m_pnMotifWidth[k]);

	//calculate P(shift|I,M,S)
	double dblCurLog=
		CalLogPCondPW(2,2,m_pnMotifWidth[k],m_pnWeightMat[k],dblLogBgPrCurSample);
	double dblNewLog=
		CalLogPCondPW(2,2,m_pnMotifWidth[k],pnNewWeightMat,dblLogBgPrNewSample);

	//MH
	bool bAccept=MH(dblCurLog,dblNewLog,0);
	if(bAccept)
	{
		delete[] m_pnWeightMat[k];
		m_pnWeightMat[k]=pnNewWeightMat;
		DelSiteHead(m_pSiteHead[k]);
		m_pSiteHead[k]=pShiftSample;
	}
	else
	{
		delete[] pnNewWeightMat;
		DelSiteHead(pShiftSample);
	}
}

void CGibbsDP::CheckBoundary(SITEINFO** pOneSiteHead,bool* pbBoundary)
{
	bool bLeftEdge=false;
	bool bRightEdge=false;
	int nMotifWidth=0;

	for(int n=0;n<m_nSequenceNumber;n++)
	{
		SITEINFO* pCurMotif=pOneSiteHead[n]->pNextSite;
		int nSeqLength=m_pstrSequence[n].GetLength();
		while(pCurMotif)
		{
			nMotifWidth=(pCurMotif->strSite).GetLength();
			if(pCurMotif->bForward)	//site on forward strand
			{
				if(pCurMotif->nPos==0)
					bLeftEdge=true;
				if(pCurMotif->nPos==nSeqLength-nMotifWidth)
					bRightEdge=true;
			}
			else
			{
				if(pCurMotif->nPos==0)
					bRightEdge=true;
				if(pCurMotif->nPos==nSeqLength-nMotifWidth)
					bLeftEdge=true;
			}
			pCurMotif=pCurMotif->pNextSite;
		}
		if(bLeftEdge&&bRightEdge)	//do not shift phase in this case
			break;
	}
	pbBoundary[0]=bLeftEdge;
	pbBoundary[1]=bRightEdge;

}

double CGibbsDP::CalLogPCondPW(int nMBCount,int nTotalCount,int nWidth,int* pnWeightMat,double dblLogBgPrCurSample)
{
	int nSumPseudo=0;
	int nTotalSite=0;
	for(int j=0;j<4;j++)
	{
		nSumPseudo+=m_pnPseudoBpCounts[j];
		nTotalSite+=pnWeightMat[j];
	}

	double dblCurLog=logGamma(nMBCount)-logGamma(nTotalCount);
	for(int i=0;i<nWidth;i++)
	{
		for(int j=0;j<4;j++)
		{
			dblCurLog+=logGamma(pnWeightMat[4*i+j]+m_pnPseudoBpCounts[j]);
		}
	}
	dblCurLog-=nWidth*logGamma(nTotalSite+nSumPseudo);
	dblCurLog-=dblLogBgPrCurSample;
	dblCurLog+=nWidth*log(15)-logGamma(nWidth+1);
	return dblCurLog;
}

bool CGibbsDP::MH(double dblCurLogPP,double dblNewLogPP,double dblLogProPRate)	//Metropolis Hasting ratio
{
	double dblr;
	double dblDiff=dblNewLogPP-dblCurLogPP+dblLogProPRate;
	bool bAccept=false;

	if(dblDiff<-200)
		bAccept=false;
	else if(dblDiff>0)
		bAccept=true;
	else
	{
		dblr=exp(dblDiff);		
		double dblU=double(rand())/RAND_MAX;
		if(dblU<dblr)
			bAccept=true;
	}
	return bAccept;
}

//operation on marginal distributions
void CGibbsDP::SetZerosMD(int** pnMA,int* pnMW,int*pnMC,int** pnMAMod,int* pnML)
{
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		//marginal distribution of modules
		int nLength=m_pstrSequence[n].GetLength();
		pnMAMod[n]=new int[nLength];
		int i=0;
		for(i=0;i<nLength;i++)
			pnMAMod[n][i]=0;

		nLength*=2;
		pnMA[n]=new int[nLength*m_nMotifNumber];
		//set all be 0
		for(i=0;i<nLength*m_nMotifNumber;i++)
			pnMA[n][i]=0;
	}
	int i=0;
	for(i=0;i<m_nMotifNumber*MAXWIDTH;i++)
		pnMW[i]=0;
	for(i=0;i<m_nMotifNumber*MAXSITENUMBER;i++)
		pnMC[i]=0;
	for(i=0;i<50;i++)
		pnML[i]=0;
}

void CGibbsDP::DeleteMD(int** pnMA,int* pnMW,int* pnMC,int** pnMAMod)	
{
	delete[] pnMW;
	delete[] pnMC;

	for(int n=0;n<m_nSequenceNumber;n++)
	{
		delete[] pnMA[n];
		delete[] pnMAMod[n];
	}
	delete[] pnMA;
	delete[] pnMAMod;
}

void CGibbsDP::RecordMD(int** pnMA,int* pnMW,int* pnMC,int** pnMAMod,int* pnML)	
{
	if(!m_bMotifMode)
	{
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			//module location
			for(int m=0;m<m_pModule[n].nCount;m++)
			{
				int nStart=m_pModule[n].pnStart[m];
				int nEnd=m_pModule[n].pnStart[m]+m_nModuleLength;
				for(int i=nStart;i<nEnd;i++)
					pnMAMod[n][i]++;
			}
		}
		int nIndex=m_nModuleLength/m_nDeltL;
		if(nIndex>=50) nIndex=49;
		pnML[nIndex]++;
	}

	for(int k=0;k<m_nMotifNumber;k++)
	{
		//site locations
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			SITEINFO* pCurSite=m_pSiteHead[k][n]->pNextSite;
			while(pCurSite)
			{
				int nPos=pCurSite->nPos;
				if(pCurSite->bForward)
					pnMA[n][2*m_nMotifNumber*nPos+2*k]++;
				else
					pnMA[n][2*m_nMotifNumber*nPos+2*k+1]++;
				pCurSite=pCurSite->pNextSite;
			}
		}
		pnMW[k*MAXWIDTH+m_pnMotifWidth[k]]++;
		if(m_pnSiteCount[k]<MAXSITENUMBER)
			pnMC[k*MAXSITENUMBER+m_pnSiteCount[k]]++;
		else
		{
			m_bOverMaxSite=true;
			pnMC[k*MAXSITENUMBER+MAXSITENUMBER-1]++;
		}
	}
}

void CGibbsDP::OutputModPr(int** pnMAMod,int* pnLength,int nSampleSize,CString strOutput)
{
	CString strFileName;
	strFileName=strOutput+"_modpr.txt";
	FILE* Output;
	Output=fopen(strFileName.m_pString,"a+");
	if(Output!=NULL)
	{
		fprintf(Output,"\n**********\n");
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			fprintf(Output,m_pstrSeqName[n].m_pString);
			fprintf(Output,"(%d)\n",pnLength[n]);
			for(int i=0;i<pnLength[n];i++)
				fprintf(Output,"%.3lf\n",double(pnMAMod[n][i])/nSampleSize);
		}
		fclose(Output);
	}

}


void CGibbsDP::SumNeighborMD(int** pnMA)	
{
	for(int k=0;k<2*m_nMotifNumber;k++)
	{
		if(!m_bReverse && k%2==1)	//did not sample reverse chain.
			continue;
		//site locations
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			int nLength=m_pstrSequence[n].GetLength();
			for(int nPos=0;nPos<nLength-2;nPos++)
			{
				if(pnMA[n][2*m_nMotifNumber*nPos+k]>0.05*(m_nMaxIteration-m_nBI))
				{
					//find the maximum score within a range.
					int nLeft=nPos-2;
					if(nLeft<0) nLeft=0;
					int nRight=nPos+6;
					if(nRight>nLength-1) nRight=nLength-1;

					int nStrand;
					int nReverseStrand;
					if(k%2==0) 
					{
						nStrand=k;
						nReverseStrand=k+1;
						if(!m_bReverse) nReverseStrand=k;
					}
					else	
					{
						nStrand=k-1;
						nReverseStrand=k;
					}

					int nMax=nPos;
					int nDir=k;
					int nMaxCount=pnMA[n][2*m_nMotifNumber*nPos+k];

					int r=0;
					for(r=nStrand;r<=nReverseStrand;r++)
					{
						for(int i=nLeft;i<nRight;i++)
						{
							if(pnMA[n][2*m_nMotifNumber*i+r]>nMaxCount)
							{
								nMaxCount=pnMA[n][2*m_nMotifNumber*i+r];
								nMax=i;
								nDir=r;
							}
						}
					}

					//put all the score on the maximum position
					int nTotalCount=0;
					for(r=nStrand;r<=nReverseStrand;r++)
					{
						for(int i=nLeft;i<nRight;i++)
						{
							nTotalCount+=pnMA[n][2*m_nMotifNumber*i+r];
							pnMA[n][2*m_nMotifNumber*i+r]=0;
						}
					}
					pnMA[n][2*m_nMotifNumber*nMax+nDir]=nTotalCount;

				}
			}
		}
	}	
}

void CGibbsDP::InferSiteCount(int* pnMC)
{
	double* pdblSmMC=new double[MAXSITENUMBER];
	int k=0;
	for(k=0;k<m_nMotifNumber;k++)
	{
		
		//smooth the histogram for sitecounts;
		int i=0;
		for(i=0;i<MAXSITENUMBER;i++)
		{
			int nLeft=i-3;
			if(nLeft<0) nLeft=0;
			int nRight=i+3;
			if(nRight>=MAXSITENUMBER) nRight=MAXSITENUMBER-1;

			pdblSmMC[i]=0;
			for(int j=nLeft;j<=nRight;j++)
			{
				pdblSmMC[i]+=pnMC[k*MAXSITENUMBER+j];
			}
			pdblSmMC[i]/=(nRight-nLeft+1);
		}

		//find the modes in smoothed histogram
		int nModes[5];
		int nIndex=0;
		for(i=0;i<MAXSITENUMBER;i++)
		{
			int nLeft=i-5;
			if(nLeft<0) nLeft=0;
			int nRight=i+5;
			if(nRight>=MAXSITENUMBER) nRight=MAXSITENUMBER-1;

			bool bMode=true;
			for(int j=nLeft;j<nRight;j++)
			{
				if(j!=i && pdblSmMC[j]>pdblSmMC[i])
				{
					bMode=false;
					break;
				}
			}

			if(bMode && pdblSmMC[i]>1)
			{
				nModes[nIndex]=i;
				nIndex++;
			}
			if(nIndex>=5) break;			
		}
		
		//finding the global modes, if is near zero, find the sub mode
		if(nIndex==0) 
		{
			m_pnSiteCount[k]=0;
			continue;
		}

		int nMaxMode=nModes[0];
		double dblMaxCount=pdblSmMC[nMaxMode];
		for(int j=1;j<nIndex;j++)
		{
			if(pdblSmMC[nModes[j]]>dblMaxCount)
			{
				nMaxMode=nModes[j];
				dblMaxCount=pdblSmMC[nMaxMode];
			}
		}

		if(nMaxMode<=4 && nIndex>1)
		{
			int nZeroMode=nMaxMode;
			//find the second largest mode to be the site count
			nMaxMode=nModes[1];
			dblMaxCount=pdblSmMC[nMaxMode];
			for(int j=2;j<nIndex;j++)
			{
				if(pdblSmMC[nModes[j]]>dblMaxCount)
				{
					nMaxMode=nModes[j];
					dblMaxCount=pdblSmMC[nMaxMode];
				}
			}

			if(4*dblMaxCount<pdblSmMC[nZeroMode])
				m_pnSiteCount[k]=nZeroMode;
			else
				m_pnSiteCount[k]=nMaxMode;

		}
		else
			m_pnSiteCount[k]=nMaxMode;

		//if the count in MAP score is reasonable use that as the site count
		if(m_pnSiteCount[k]<=4)
		{
			int nMAPCount=CalWeightMat(m_pOptSiteHead[k],m_pnWeightMat[k],m_pnMotifWidth[k]);
			if(nMAPCount<m_nSequenceNumber*2)
				m_pnSiteCount[k]=nMAPCount;
		}
	}

	delete[] pdblSmMC;

	//modify by FP factor
	for(k=0;k<m_nMotifNumber;k++)
		m_pnSiteCount[k]=int(m_pnSiteCount[k]*0.9+0.5); 
}

void CGibbsDP::InferModule(int** pnModMA,int* pnLength)
{
	m_nTotModLength=0;
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		for(int i=0;i<pnLength[n];i++)
		{
			if(pnModMA[n][i]>(m_nMaxIteration-m_nBI)/2)
				m_nTotModLength++;
		}
	}
	m_nModuleCount=m_nTotModLength/m_nModuleLength;
}


void CGibbsDP::FindOptimum(int** pnMA,int* pnMW,int nIteration,bool bControlFP)
{
	struct SITECOUNT
	{
		int nCount;
		SITECOUNT* pNextCount;
	};

	SITECOUNT* pOrderSiteCountHead=new SITECOUNT;
	pOrderSiteCountHead->pNextCount=NULL;


	//find the optimal with for each motif
	for(int k=0;k<m_nMotifNumber;k++)
	{
/*		m_pnMotifWidth[k]=0;
		int nMaxPP=pnMW[k*MAXWIDTH];
		for(int i=1;i<MAXWIDTH;i++)
		{
			if(pnMW[k*MAXWIDTH+i]>nMaxPP)
			{
				m_pnMotifWidth[k]=i;
				nMaxPP=pnMW[k*MAXWIDTH+i];
			}
		}*/

		m_pnMotifWidth[k]=0;
		for(int w=m_nMinWidth;w<=m_nMaxWidth;w++)
			m_pnMotifWidth[k]+=w*pnMW[k*MAXWIDTH+w];
		m_pnMotifWidth[k]=int(m_pnMotifWidth[k]/double(nIteration)+0.5);

		DelSiteHead(m_pOptSiteHead[k]);
		m_pOptSiteHead[k]=new SITEINFO*[m_nSequenceNumber];
		int n=0;
		for(n=0;n<m_nSequenceNumber;n++)
		{
			m_pOptSiteHead[k][n]=new SITEINFO;
			m_pOptSiteHead[k][n]->pNextSite=NULL;
		}

		//find the count for the sitecount ranked sites
/*		if(m_pnSiteCount[k]==0) continue;

		int nMinCount=0;
		int nSiteNumber=0;
		for(n=0;n<m_nSequenceNumber;n++)
		{
			int nLength=m_pstrSequence[n].GetLength();
			for(int i=0;i<nLength;i++)
			{
				for(int r=0;r<2;r++)	//consider reverse strand
				{
					int nCurPosCount=pnMA[n][2*m_nMotifNumber*i+2*k+r];
					bool bAddSite=false;
					if(nSiteNumber<m_pnSiteCount[k] && nCurPosCount>0)
						bAddSite=true;
					else if(nCurPosCount>nMinCount)
						bAddSite=true;
					
					if(bAddSite)
					{
						nSiteNumber++;
						SITECOUNT* pCur=pOrderSiteCountHead;
						while(pCur->pNextCount)
						{
							if(pCur->pNextCount->nCount>nCurPosCount)
								break;
							else
								pCur=pCur->pNextCount;
						}
						SITECOUNT* pNew=new SITECOUNT;
						pNew->nCount=nCurPosCount;
						pNew->pNextCount=pCur->pNextCount;
						pCur->pNextCount=pNew;
						//update the minimal count
						while(nSiteNumber>m_pnSiteCount[k])
						{
							nSiteNumber--;
							pCur=pOrderSiteCountHead->pNextCount;
							pOrderSiteCountHead->pNextCount=pCur->pNextCount;
							delete pCur;
						}
						if(pOrderSiteCountHead->pNextCount)
							nMinCount=pOrderSiteCountHead->pNextCount->nCount;
						else
							nMinCount=0;


						if(!m_bReverse) break;
					}
				}//for (r
			}//for(i
		}//for(n

		//using the nMinCount to select the output sites
		if(bControlFP)
		{
			int nCutoff=int(nIteration*m_dblMotifCutoff);
			if(nMinCount<nCutoff) nMinCount=nCutoff;
		}*/

		int nMinCount=int(nIteration*m_dblMotifCutoff);

		for(n=0;n<m_nSequenceNumber;n++)
		{
			int nLength=m_pstrSequence[n].GetLength()-m_pnMotifWidth[k]+1;
			for(int i=0;i<nLength;i++)
			{
				if(pnMA[n][2*m_nMotifNumber*i+2*k]>=nMinCount)	//treated as sites
				{
					SITEINFO* pNewSite=new SITEINFO;
					pNewSite->nPos=i;
					pNewSite->bForward=true;
					pNewSite->strSite=m_pstrSequence[n].Mid(i,m_pnMotifWidth[k]);
					pNewSite->nMA=pnMA[n][2*m_nMotifNumber*i+2*k];
					pNewSite->pNextSite=NULL;
					AddSite(m_pOptSiteHead[k][n],pNewSite);
				}
				else if(pnMA[n][2*m_nMotifNumber*i+2*k+1]>=nMinCount)	//treated as sites
				{
					SITEINFO* pNewSite=new SITEINFO;
					pNewSite->nPos=i;
					pNewSite->bForward=false;
					pNewSite->strSite=Reverse(m_pstrSequence[n].Mid(i,m_pnMotifWidth[k]));
					pNewSite->nMA=pnMA[n][2*m_nMotifNumber*i+2*k+1];
					pNewSite->pNextSite=NULL;
					AddSite(m_pOptSiteHead[k][n],pNewSite);
				}
			}
		}

		//delete ordersitehead
		SITECOUNT* pCur=pOrderSiteCountHead->pNextCount;
		SITECOUNT* pDel;
		while(pCur)
		{
			pDel=pCur;
			pCur=pCur->pNextCount;
			delete pDel;
		}
		pOrderSiteCountHead->pNextCount=NULL;
	}

	delete pOrderSiteCountHead;

}

void CGibbsDP::AlignSites(int** pnMA)
{
	for(int k=0;k<m_nMotifNumber;k++)
	{
		double* pdblTheta=new double[m_pnMotifWidth[k]*4];
		m_pnSiteCount[k]=CalWeightMat(m_pOptSiteHead[k],m_pnWeightMat[k],m_pnMotifWidth[k]);
		CopySample(m_pOptSiteHead[k],m_pSiteHead[k]);
		if(m_pnSiteCount[k]<=2)
		{
			continue;
		}

		//record the original positions of the sites and check the max site marginal pr.
		int nMaxMA=0;
		int* pnOrPos=new int[m_pnSiteCount[k]];
		int nSiteIndex=0;
		double dblLogBg=0;
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			SITEINFO* pCurSite=m_pOptSiteHead[k][n]->pNextSite;
			int nSeqLength=m_pstrSequence[n].GetLength();
			while(pCurSite)
			{
				dblLogBg+=CalLogBgPr(pCurSite->strSite);
				pnOrPos[nSiteIndex]=pCurSite->nPos;
				if(pCurSite->nMA>nMaxMA)
					nMaxMA=pCurSite->nMA;
				nSiteIndex++;
				pCurSite=pCurSite->pNextSite;
			}
		}
		//if max marginal pr < 0.5 then delete the sites
		if(nMaxMA<(m_nMaxIteration-m_nBI)/2)
		{
			m_pnSiteCount[k]=1;
			continue;
		}
		double dblMaxScore=CalFixSiteScore(m_pnWeightMat[k],m_pnMotifWidth[k],dblLogBg);


		for(int nIteration=0;nIteration<20;nIteration++)
		{
			//get the weight matrix
			for(int i=0;i<m_pnMotifWidth[k]*4;i++)
				pdblTheta[i]=(m_pnWeightMat[k][i]+1.0)/(m_pnSiteCount[k]+4.0);
			dblLogBg=0;
			int nSiteIndex=0;
			for(int n=0;n<m_nSequenceNumber;n++)
			{
				SITEINFO* pCurSite=m_pOptSiteHead[k][n]->pNextSite;
				int nSeqLength=m_pstrSequence[n].GetLength();
				while(pCurSite)
				{
					//get the flanking region
					int nLeft=pnOrPos[nSiteIndex]-5;
					if(nLeft<0) nLeft=0;
					int nRight=pnOrPos[nSiteIndex]+5;
					if(nRight+m_pnMotifWidth[k]>nSeqLength) nRight=nSeqLength-m_pnMotifWidth[k];
					
					//score all the sites
					int nCount=nRight-nLeft+1;
					double* pdblScore=new double[nCount*2];
					CString strWindow;
					int j=0;
					for(int i=nLeft;i<=nRight;i++)
					{
						strWindow=m_pstrSequence[n].Mid(i,m_pnMotifWidth[k]);
						pdblScore[j]=-CalLogBgPr(strWindow);
						pdblScore[j]+=CalLogMotifPr(strWindow,pdblTheta);
						j++;
						pdblScore[j]=-CalLogBgPr(strWindow);
						if(!m_bReverse)
							pdblScore[j]=-2000;
						else
						{
							strWindow=Reverse(strWindow);
							pdblScore[j]+=CalLogMotifPr(strWindow,pdblTheta);
						}
						j++;
					}
					
					//get the maximum score and normalize by that
					double dblSum=0;
					int nMax=Maximum(pdblScore,nCount*2);
					for(j=0;j<nCount*2;j++)
					{
						pdblScore[j]-=pdblScore[nMax];
						if(pdblScore[j]>-200)
							pdblScore[j]=dblSum+exp(pdblScore[j]);
						else
							pdblScore[j]=dblSum;
						dblSum=pdblScore[j];
					}
					pdblScore[nCount*2-1]+=0.1;
					//sample one of the position to be the site
					int nSelect=0;
					double dblU=double(rand())/(RAND_MAX+1)*dblSum;
					for(j=0;j<nCount*2;j++)
					{
						if(dblU<pdblScore[j])
						{
							nSelect=j;
							break;
						}
					}

					pCurSite->nPos=nLeft+nSelect/2;
					strWindow=m_pstrSequence[n].Mid(pCurSite->nPos,m_pnMotifWidth[k]);
					dblLogBg+=CalLogBgPr(strWindow);
					if(nMax%2==0)
					{
						pCurSite->bForward=true;
						pCurSite->strSite=strWindow;
					}
					else
					{
						pCurSite->bForward=false;
						pCurSite->strSite=Reverse(strWindow);
					}

					pCurSite=pCurSite->pNextSite;
					nSiteIndex++;
				}//while
			}//for(int n

			//calcuate the Bayesian score the sites
			CalWeightMat(m_pOptSiteHead[k],m_pnWeightMat[k],m_pnMotifWidth[k]);
			double dblScore=CalFixSiteScore(m_pnWeightMat[k],m_pnMotifWidth[k],dblLogBg);
			if(dblScore>dblMaxScore)
			{
				dblMaxScore=dblScore;
				CopySample(m_pOptSiteHead[k],m_pSiteHead[k]);
			}
		}//for(nIteration
		delete[] pdblTheta;
		delete[] pnOrPos;

	}
	CopySample(m_pSiteHead,m_pOptSiteHead);
}

double CGibbsDP::ISforBF(double* pr,double* q,double** Theta,int* pnSeqLength)	//importance sampling for Bayes factor
{

	//get sequence lengths and the probability of positions
	//calculate null model probability: K=0
	double dblLogPrNull=CalLogNullModel();

	double** pdblLogPrPos=new double*[m_nSequenceNumber];
	for(int n=0;n<m_nSequenceNumber;n++)
	{
		pdblLogPrPos[n]=new double[pnSeqLength[n]*(m_nMotifNumber*2+1)];
	}	

	//shrink the counts to diverge the parameter distributions
	double dblLumda=0.3;
	double dblLogPropConst=ShrinkCount(dblLumda);
	double dblLogPrior=logGamma(m_nMotifNumber+1);
	for(int k=0;k<m_nMotifNumber;k++)
	{
		dblLogPrior+=logGamma(4)*m_pnMotifWidth[k];
	}

	int nSampleSize=100;//1000;
	int nIteration=0;

	double* dblLogWeight=new double[nSampleSize];
	double* dblLogFW=new double[nSampleSize];
	while(nIteration<nSampleSize)
	{
		//proposal
		double dblLogProp=ProposeParameter(pr,q,Theta);
		CalLogPrPos(pdblLogPrPos,pnSeqLength,Theta);

		double dblLogPSGP=0;
		for(int n=0;n<m_nSequenceNumber;n++)
		{
			dblLogPSGP+=ProbSeqGParam(n,*pr,q,Theta,pdblLogPrPos[n]);
		}	
		
		dblLogWeight[nIteration]=-dblLogProp;
		dblLogFW[nIteration]=dblLogPSGP-dblLogProp;
		nIteration++;
	}
	delete[] pdblLogPrPos;

	//sum of weights
	double dblMaxWt=dblLogWeight[Maximum(dblLogWeight,nSampleSize)];
	double dblSumWt=0;
	int i=0;
	for(i=0;i<nSampleSize;i++)
	{
		double dblNormWt=dblLogWeight[i]-dblMaxWt;
		if(dblNormWt<-300)
			dblSumWt+=0;
		else
			dblSumWt+=exp(dblNormWt);
	}
	dblSumWt=dblMaxWt+log(dblSumWt);

	//sum of P(S|.)*weights
	double dblMaxFW=dblLogFW[Maximum(dblLogFW,nSampleSize)];
	double dblSumFW=0;
	for(i=0;i<nSampleSize;i++)
	{
		double dblNormWt=dblLogFW[i]-dblMaxFW;
		if(dblNormWt<-300)
			dblSumFW+=0;
		else
			dblSumFW+=exp(dblNormWt);
	}
	dblSumFW=dblMaxFW+log(dblSumFW);


	delete[] dblLogWeight;
	delete[] dblLogFW;

	m_dblEstBF=dblSumFW-dblSumWt-dblLogPrNull-m_nMotifNumber*log(3);

	return m_dblEstBF;
}

double CGibbsDP::ShrinkCount(double dblLumda)
{
	double dblLogNormConst=0;
	int nSumCount=0;

	//for weight matrix
	int k=0;
	for(k=0;k<m_nMotifNumber;k++)
	{
		for(int i=0;i<m_pnMotifWidth[k];i++)
		{
			double nTotRowCount=0;
			int j=0;
			for(j=0;j<4;j++)
				nTotRowCount+=m_pnWeightMat[k][i*4+j];
			//shrink and add pseudo-count
			nSumCount=0;
			for(j=0;j<4;j++)
			{
				//normalize
				double dblNCount=(m_pnWeightMat[k][i*4+j]/(nTotRowCount+0.001))*m_pnSiteCount[k];
				m_pnWeightMat[k][i*4+j]=int(dblNCount*dblLumda)+1;

				//calculate normalizing constant
				dblLogNormConst-=logGamma(m_pnWeightMat[k][i*4+j]);
				nSumCount+=m_pnWeightMat[k][i*4+j];
			}
			dblLogNormConst+=logGamma(nSumCount);
		}
	}

	//for q
	m_pnSiteCount[m_nMotifNumber]=m_nModuleCount*m_nModuleLength;
	for(k=0;k<m_nMotifNumber;k++)
		m_pnSiteCount[m_nMotifNumber]-=m_pnSiteCount[k]*m_pnMotifWidth[k];
	
	nSumCount=0;
	for(k=0;k<=m_nMotifNumber;k++)
	{
		m_pnSiteCount[k]=int(m_pnSiteCount[k]*dblLumda)+1;
		dblLogNormConst-=logGamma(m_pnSiteCount[k]);
		nSumCount+=m_pnSiteCount[k];
	}
	dblLogNormConst+=logGamma(nSumCount);

	//for r
	m_nTotalLength=int((m_nTotalLength-m_nModuleCount*m_nModuleLength)*dblLumda)+1;
	m_nModuleCount=int(m_nModuleCount*dblLumda)+1;
	dblLogNormConst+=logGamma(m_nModuleCount+m_nTotalLength);
	dblLogNormConst-=(logGamma(m_nModuleCount)+logGamma(m_nTotalLength));

	return dblLogNormConst;
}

double CGibbsDP::ProposeParameter(double* pr,double* q,double** Theta)	//generate parameters from proposal
{
	double dblLogPr=0;
	CStochasticDistribution RandNumber;
	//r
	*pr=RandNumber.Beta(m_nModuleCount,m_nTotalLength);
	dblLogPr+=m_nModuleCount*log(*pr+1e-200)+m_nTotalLength*log(1-(*pr)+1e-200);

	//2) q_n
	RandNumber.Dirichlet(m_pnSiteCount,m_nMotifNumber+1,q);
	int k=0;
	for(k=0;k<m_nMotifNumber+1;k++)
		dblLogPr+=m_pnSiteCount[k]*log(q[k]+1e-200);

	//calculate theta_n
	int* pnBpCount=new int[sizeof(int)*4];
	for(k=0;k<m_nMotifNumber;k++)
	{
		for(int i=0;i<m_pnMotifWidth[k];i++)
		{
			int j=0;
			for(j=0;j<4;j++)
			{
				pnBpCount[j]=m_pnWeightMat[k][i*4+j];
			}
			RandNumber.Dirichlet(pnBpCount,4,&(Theta[k][i*4]));
			for(j=0;j<4;j++)
				dblLogPr+=pnBpCount[j]*log(Theta[k][i*4+j]+1e-200);
		}
	}
	delete[] pnBpCount;
	return dblLogPr;
}

double CGibbsDP::ProbSeqGParam(int n,double r_n,double* q_n,double** Theta_n,double* pdblLogPrPos)	//P(S|\psi) for the nth seq
{
	int nLength=m_pstrSequence[n].GetLength();

	//calculate the sequence probability under the module model
	double* pdblLogPrModuleCurPos=new double[sizeof(double)*nLength];
	CalModuleLogPr(nLength-1,nLength,pdblLogPrPos,q_n,pdblLogPrModuleCurPos);

	//Calculate module probability and forward summation of Module
	double* pModuleLogPr=new double[sizeof(double)*nLength];
	double* pSumModuleLogPr=new double[sizeof(double)*nLength];
	int nModuleLength=m_nModuleLength;
	if(nModuleLength>nLength) nModuleLength=nLength;
	
	double dblSum=0;
	int i=0;
	for(i=0;i<nModuleLength-1;i++)
	{
		pModuleLogPr[i]=pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
		dblSum+=pModuleLogPr[i];//+log(1-r_n);
		pSumModuleLogPr[i]=dblSum;
	}
	for(i=nModuleLength-1;i<nLength;i++)
	{
		if(i>=nModuleLength)
			pModuleLogPr[i]=pdblLogPrModuleCurPos[i]-pdblLogPrModuleCurPos[i-nModuleLength];
		else
			pModuleLogPr[i]=pdblLogPrModuleCurPos[i];
		double dblLogP1=pSumModuleLogPr[i-1]+log(1-r_n)+pdblLogPrPos[i*(2*m_nMotifNumber+1)+2*m_nMotifNumber];
		double dblLogP2;
		if(i==nModuleLength-1)
			dblLogP2=log(r_n)+pModuleLogPr[i];
		else
			dblLogP2=pSumModuleLogPr[i-nModuleLength]+log(r_n)+pModuleLogPr[i];
		pSumModuleLogPr[i]=LogSum(dblLogP1,dblLogP2);
	}

	//record P(S|\psi,H_K)
	double dblPSGP=pSumModuleLogPr[nLength-1];

	delete[] pModuleLogPr;
	delete[] pSumModuleLogPr;	
	delete[] pdblLogPrModuleCurPos;

	return dblPSGP;
}

double CGibbsDP::CalLogNullModel()
{
	double dblLogPrNull=0;
	for(int n=0;n<this->m_nSequenceNumber;n++)
		dblLogPrNull+=CalLogBgPr(m_pstrSequence[n]);
	return dblLogPrNull;
}

void CGibbsDP::Output(CString strOutput,int** pnMA,int** pnMAMod,int* pnSeqLength)				//output results
{
	CString	   strLine;
	char	   strTemp[1000];
	int nCutoffMod=(m_nMaxIteration-m_nBI)/2;

	//get motif information
	bool* pbFoundSite=new bool[m_nMotifNumber];
	for(int k=0;k<m_nMotifNumber;k++)
	{
		//find the width of the sites
		pbFoundSite[k]=false;
		int n=0;
		for(n=0;n<m_nSequenceNumber;n++)
		{
			SITEINFO* pCurrentMotif=m_pOptSiteHead[k][n]->pNextSite;
			if(pCurrentMotif!=NULL)
			{
				m_pnMotifWidth[k]=pCurrentMotif->strSite.GetLength();
				pbFoundSite[k]=true;
				break;
			}
		}
		if(!pbFoundSite[k])
		{
			m_pnSiteCount[k]=0;
			continue;
		}

		delete[] m_pnWeightMat[k];
		m_pnWeightMat[k]=new int[4*m_pnMotifWidth[k]];
		m_pnSiteCount[k]=CalWeightMat(m_pOptSiteHead[k],m_pnWeightMat[k],m_pnMotifWidth[k]);
	}

	int nTotModuleCount=0;
	int n=0;
	for(n=0;n<m_nSequenceNumber;n++)
	{
		int* pnSitePos=new int[pnSeqLength[n]];
		int i=0;
		for(i=0;i<pnSeqLength[n];i++)
			pnSitePos[i]=0;
		
		//get all the site positions for K motifs
		for(int k=0;k<m_nMotifNumber;k++)
		{
			if(m_pnSiteCount[k]<=2) continue;
			SITEINFO* pCurSite=m_pOptSiteHead[k][n]->pNextSite;
			while(pCurSite)
			{
				int nPos=pCurSite->nPos;
				pnSitePos[nPos]=k+1;
				if(!(pCurSite->bForward)) pnSitePos[nPos]=-(k+1);
				pCurSite=pCurSite->pNextSite;
			}
		}

		//get total module counts and their boundaries
		int nModuleCount=0;
		int* pnModulebound=new int[sizeof(int)*(pnSeqLength[n]/m_nModuleLength)];
		bool bModule=false;
		for(i=0;i<pnSeqLength[n];i++)
		{
			if(!bModule && pnMAMod[n][i]>nCutoffMod)
			{
				bModule=true;
				pnModulebound[nModuleCount*2]=i;
			}
			else if(bModule && pnMAMod[n][i]<nCutoffMod)
			{
				bModule=false;
				pnModulebound[nModuleCount*2+1]=i;
				nModuleCount++;
			}
		}

		//write out the modules
		for(int m=0;m<nModuleCount;m++)
		{
			int nStart=pnModulebound[2*m];
			int nEnd=pnModulebound[2*m+1];
			if(nEnd-nStart<m_nModuleLength/5) continue;

			int nSiteCount=0;
			CString strStruct;
			for(i=nStart;i<=nEnd;i++)
			{
				if(pnSitePos[i]>0)
				{
					sprintf(strTemp,"+%d(%d)",pnSitePos[i],i);
					strStruct=strStruct+strTemp;
					nSiteCount++;
				}
				else if(pnSitePos[i]<0)
				{
					sprintf(strTemp,"%d(%d)",pnSitePos[i],i);
					strStruct=strStruct+strTemp;
					nSiteCount++;
				}
			}
			if(nSiteCount==0) continue;
			else 
			{
				strLine=strLine+m_pstrSeqName[n];
				sprintf(strTemp,"\t[%d,%d]\t",nStart,nEnd);
				strLine=strLine+strTemp+strStruct+"\n";
				nTotModuleCount++;
			}
		}
		delete[] pnModulebound;
		delete[] pnSitePos;
	}
	m_nModuleCount=nTotModuleCount;

	//write the output file
	FILE* Output;
	Output=fopen(strOutput.m_pString,"a+");
	if(Output!=NULL)
	{
		fprintf(Output,m_strInformation.m_pString);
		fprintf(Output,"\n\n");
		if(m_bOutputBF)
		{
			fprintf(Output,"\nLog-PMO(K=%d)=%.2lf\n\n",m_nMotifNumber,m_dblEstBF);
		}

		if(!m_bMotifMode)
		{
			fprintf(Output,"\n\n****************\n# of Modules:%d\n",nTotModuleCount);
			if(nTotModuleCount>0)
				fprintf(Output,strLine.m_pString);
		}

		//write motif sites
		for(int k=0;k<m_nMotifNumber;k++)
		{
			fprintf(Output,"\n\n\n**********************\nMotif_%d:\n",k+1);

			if(m_pnSiteCount[k]<=2 || !pbFoundSite[k])
			{
				fprintf(Output,"No sites found!\n\n");
				continue;
			}

			//write summary stat
			fprintf(Output,"number of sites: %d\n\tA\tC\tG\tT\tCon\n",m_pnSiteCount[k]);

			char cCon[4]={'A','C','G','T'};
			for(int i=0;i<m_pnMotifWidth[k];i++)
			{
				double	dblCon=0;
				int		nCon=0;
				sprintf(strTemp,"%d\t",i+1);
				strLine=strTemp;
				for(int j=0;j<4;j++)
				{
					if(m_pnWeightMat[k][i*4+j]>dblCon)
					{
						nCon=j;
						dblCon=m_pnWeightMat[k][i*4+j];
					}
					sprintf(strTemp,"%.3lf\t",double(m_pnWeightMat[k][i*4+j])/m_pnSiteCount[k]);
					strLine=strLine+strTemp;
				}
				strLine=strLine+cCon[nCon]+"\n";
				fprintf(Output,strLine.m_pString);
			}

			//write sites that found
			int nSeqLength=m_nTotalLength/m_nSequenceNumber;
			CString strPos;

			for(n=0;n<m_nSequenceNumber;n++)
			{
				SITEINFO* pCurrentMotif=m_pOptSiteHead[k][n]->pNextSite;
				while(pCurrentMotif)
				{
					strLine=m_pstrSeqName[n]+"\t";
					double dblPP=double(pCurrentMotif->nMA)/(m_nMaxIteration-m_nBI);
					if(pCurrentMotif->bForward)
						strLine=strLine+"f\t";
					else
						strLine=strLine+"r\t";
					sprintf(strTemp,"%d\t",pCurrentMotif->nPos);
					strLine=strLine+strTemp;
					sprintf(strTemp,"\t%.3lf\t\n",dblPP);
					strLine=strLine+strTemp;
					fprintf(Output,strLine.m_pString);

					strLine=pCurrentMotif->strSite+"\n";
					fprintf(Output,strLine.m_pString);

					pCurrentMotif=pCurrentMotif->pNextSite;
				}
			}
		}
		fclose(Output);
	}

	delete[] pbFoundSite;
}

//debug purpose only
void CGibbsDP::OutputSite()							//output sites in simple form
{
	FILE* Output;
	for(int k=0;k<m_nMotifNumber;k++)
	{
		char strTemp[50];
		sprintf(strTemp,"motif%d",k+1);
		CString strFileName=m_strOutput+strTemp;
		Output=fopen(strFileName.m_pString,"a+");
		if(Output)
		{
			for(int n=0;n<m_nSequenceNumber;n++)
			{
				SITEINFO* pCurrentMotif=m_pOptSiteHead[k][n]->pNextSite;
				while(pCurrentMotif)
				{
					double dblPP=double(pCurrentMotif->nMA)/(m_nMaxIteration-m_nBI);
					int nOrientation=1;
					if(!pCurrentMotif->bForward)
						nOrientation=-1;
					fprintf(Output,"%d\t%d\t%d\t%.3lf\n",n,pCurrentMotif->nPos,nOrientation,dblPP);
					pCurrentMotif=pCurrentMotif->pNextSite;
				}
			}
			fclose(Output);
		}
	}
}


/*-------------friend function-----------------------*/
int intpow(int nBase,int nPower)	//calculate interger power
{
	int nResult=1;
	for(int i=0;i<nPower;i++)
	{
		nResult*=nBase;
	}
	return nResult;
}

CString Reverse(CString strSeq)					//reverse the sequence
{
	CString strReverse=strSeq;
	int		nLength=strSeq.GetLength();
	for(int i=0;i<nLength;i++)
	{
		switch(strSeq[i])
		{
		case 'A':
			strReverse.SetAt(nLength-1-i,'T');
			break;
		case 'T':
			strReverse.SetAt(nLength-1-i,'A');
			break;
		case 'G':
			strReverse.SetAt(nLength-1-i,'C');
			break;
		case 'C':
			strReverse.SetAt(nLength-1-i,'G');
			break;
		case 'S':
			strReverse.SetAt(nLength-1-i,'S');
			break;
		case 'R':
			strReverse.SetAt(nLength-1-i,'Y');
			break;
		case 'Y':
			strReverse.SetAt(nLength-1-i,'R');
			break;
		case 'W':
			strReverse.SetAt(nLength-1-i,'W');
			break;
		default:
			strReverse.SetAt(nLength-1-i,strSeq[i]);
		}
	}
	return strReverse;
}

//code the word
int WordCode(CString strWord)
{
	int nLength=strWord.GetLength();
	int nCode=0;
	for(int i=0;i<nLength;i++)
	{
		switch(strWord[i])
		{
		case 'A':
			nCode+=0;
			break;
		case 'C':
			nCode+=1*intpow(4,nLength-i-1);
			break;
		case 'G':
			nCode+=2*intpow(4,nLength-i-1);
			break;
		case 'T':
			nCode+=3*intpow(4,nLength-i-1);
			break;
		}
	}
	return nCode;
}

int Sum(int* pVector,int nSize)
{
	int dblSum=0;
	for(int d=0;d<nSize;d++)
	{
		dblSum+=pVector[d];
	}
	return dblSum;
}

double logGamma(int nX)				//log-gamma function of integer input
{
	if(nX==1) return 0;
	double nY=0;
	for(int j=nX-1;j>1;j--)
		nY+=log(j);
	return nY;
}

int	Minimum(int* pVector,int nSize)
{
	int nMin=0;
	int nMinValue=pVector[0];
	for(int d=1;d<nSize;d++)
	{
		if(pVector[d]<nMinValue)
		{
			nMinValue=pVector[d];
			nMin=d;
		}
	}
	return nMin;
}

int	Maximum(double* pVector,int nSize)
{
	int nMax=0;
	double dblMax=pVector[0];
	for(int d=1;d<nSize;d++)
	{
		if(pVector[d]>dblMax)
		{
			dblMax=pVector[d];
			nMax=d;
		}
	}
	return nMax;
}

//read one row of data
CString ReadData(CString strLine,int nSampleStart,int nSampleNumber,double* pdblData)		
{
	//read gene ID and name
	int nTab=-1;
	for(int i=0;i<nSampleStart;i++)
	{
		nTab=strLine.Find('\t',nTab+1);
	}
	CString strName;
	strName="";
	if(nTab>0)
		strName=strLine.Left(nTab);

	int nEnd=0;
	int j=0;
	CString strData;
	for(j=0;j<nSampleNumber;j++)
	{
		nEnd=strLine.Find('\t',nTab+1);
		if(nEnd<0) 
		{
			nEnd=strLine.GetLength();
		}
		strData=strLine.Mid(nTab+1,nEnd-nTab-1);
		pdblData[j]=atof(strData.m_pString);
		nTab=nEnd;
	}

	return strName;
}