////////////////////////////////////////////////////////////////
//class of the computing core of CisModule project
#include "MyString.h"

struct SITEINFO		//collection of sites for one motif
{
	CString strSite;
	int		nPos;
	bool	bForward;
	int		nMA;
	SITEINFO* pNextSite;
};

struct MODULEINFO	//information for the modules on one sequence
{
	int		nCount;		//how many modules
	int*	pnStart;	//where are they
	double  dblLogModScore;	//log Module scores
	double	dblLogMLCratio;	//log pr of the changed part score
};


class CGibbsDP  
{
public:
	int				m_nMotifNumber;		//# of motifs: K
	int				m_nModuleLength;	//length of each module: l
	int				m_nModuleCount;		//total number of modules
	int				m_nTotModLength;	//infered total module length
	bool			m_bFixMLength;		//fix module length
	int				m_nDeltL;			//step size for module length change
	SITEINFO***		m_pSiteHead;		//head pointer to each kind of motif site
	MODULEINFO*		m_pModule;			//modules for each sequence
	int**			m_pnWeightMat;		//weight matrices for all the motifs
	int*			m_pnMotifWidth;		//width for each motif
	bool			m_bFixWidth;		//fix width or not
	int				m_nMaxWidth;		//maximal allowable width
	int				m_nMinWidth;		//minimal allowable width
	int*			m_pnSiteCount;		//number of site for each motif
	double			m_dblMotifCutoff;	//cut off value of motif sites
	bool			m_bMotifMode;		//degenerate to K-motif sampler
	bool			m_bSampleParameters;	//sample parameters or simply updated by posterior mean
	bool			m_bInputBackground;		//indicate input pre-calculated background or not
	
	//cumulate stronger prior for next step of module sampling
	bool			m_bPriorWM;			//have prior information for weight matrix
	int**			m_pnInitialMat;		//initial weight matrices for motifs

	CString*	m_pstrSequence;					//sets of input sequences
	CString*	m_pstrSeqName;					//names of the sequences ">****"
	CString		m_strPWMFile;					//initial values for some PWMs
	bool		m_bInitalPWM;					//use initial PWM
	int			m_nSequenceNumber;				//number of sequences
	CString		m_strOutput;					//output file
	CString		m_strInputSeq;					//input sequence file
	CString		m_strInputBgfile;				//file name for the input background pr.
	CString		m_strInputPrior;				//input file for prior
	double		m_dblBackGround[20];			//background transition probability

	SITEINFO***		m_pOptSiteHead;				//optimal sample
	double			m_dblOptLogJPP;				//optimal log posterior probability
	bool			m_bOutputBF;				//including output Posterior model odds
	double			m_dblEstBF;					//logBF(H_K;H_0)
	double			m_dblMAPLHK;				//MAP of likelihood P(S|\psi,H_K)

	bool			m_bReverse;				//sample backward or not
	int				m_nTotalLength;			//total length of the sequence (-module length)
	int				m_nMaxIteration;		//maximal iterations
	int				m_nBI;					//burnin up iterations
	bool			m_bOverMaxSite;			//found more than MAXSITENUMBER of sites

	//pseudoCounts: prior
	int				m_pnPseudoModuleCounts[2];		//(a,b)
	int*			m_pnPseudoSiteCounts;		// \alpha(0,1,...,K)
	int				m_pnPseudoBpCounts[4];		// \beta(kj);

	//error control
	bool			m_bStopRun;
	CString			m_strInformation;

public:
	CGibbsDP();
	virtual ~CGibbsDP();

	void FreeMemery();					//free all the memory

	void InputParameter(CString);		//input parameters
	void ReadInputSeq();				//read in seqeunces
	void Initialize();					//initialize member variables
	void ReadBackground(CString);		//read in background
	void BuildBackground();				//build background by the input seq.
	double CalLogNullModel();				//calculate null model probability
	double CalLogBgPr(CString);			//Calculate log background probabilities
	double CalLogMotifPr(CString,double* ); //calculate the log pr of motifs
	void Output(CString,int**,int**,int*);				//output results
	void OutputModule(CString,int**,int*);		//output modules and their structure

	int  CalWeightMat(SITEINFO**,int*,int);				//calculate marginal weight matrix
	int  CountModule(MODULEINFO*);		//count the number of modules in the seq set
	void AddSite(SITEINFO*,SITEINFO*);	//add the site in current sequence to the sitelink
	void DelSiteHead(SITEINFO**);		//delete sitehead of one motif
	void FilterSite(SITEINFO**,double);	//filter sites by their nMA
	void RandPickSite();				//pick one site from each sequence randomly
	void SampleParameters(double*,double*,double**);	//sample parameters given module/sites
	void UpdateParameters(double*,double*,double**);	//update parameters
	void UpdateSite();					//update the site through module sampling
	void ExcludeCurSeq(int);			//exclude current sequence
	void CalLogPrPos(double**,int*,double**);		//calculate the log probability for each position 
	void CalLogBgPrPos(double**,int*);				//calculate log background pr for each position
	void ApproxSampleCurSeq(int,double,double*,double**,double*,int*);		//sample the current sequence approximately
	void SampleModule(double*,double*,double,int,double*,int,int*,double*);			//sample module on current sequence
	void SampleSites(double*,double*,int,int,int);							//sample sites given module
	double CalModuleLogPr(int,int,double*,double*,double*);					//calculate the log pr of module
	double CalLogPostPrSample();								//calculate the log posterior probability of the sample
	double CalBayesianScore(int,int*,int**,int*,double);			//calculate the bayesian score for the imputed sites
	double CalFixSiteScore(int*,int,double);
	double CalLogPCondPW(int,int,int,int*,double);			//calculate the posterior conditional probability for w
	void CheckBoundary(SITEINFO**,bool*);		//check if any sites at the boundary of the sequence
	void PhaseShift(int);				//phase shift proposal of the current chain
	void UpdateWidth(int);				//sampling the motif width given current sites
	void UpdateModuleLength(int);		//sampling the module length given current modules
	bool MH(double,double,double);		//Metropolis Hasting ratio
	void CopySample(SITEINFO***,SITEINFO***);		//copy the site info
	void CopySample(SITEINFO** pSourceSites,SITEINFO** pDestSites);		//copy the site info

	//estimate BF of H_K over H_0
	double EstimateBF(double,double);			//estimate Bayes factor by MAP 
	void InputforBFK(CString);					//input parameters for calculating BF
	double ISforBF(double*,double*,double**,int*);		//importance sampling for Bayes factor
	double ShrinkCount(double);						//shrink the counts to diverge the proposal
	double ProposeParameter(double*,double*,double**);	//generate parameters from proposal
	double ProbSeqGParam(int,double,double*,double**,double*);	//P(S|\psi) for the nth seq

	void DataAugmentation();				//sampling both parameters and module\site

	//input initial PWM for partial motifs
	void ReadInputPWM(double**);				//read in the PWMs

	//online information
	void OutputInfo(int);					//output some information

	//debug purpose only
	void SetInitialParam(double**);					//set initial parameters: debug usage
	void OutputParam(double,double*,double**);		//output parameters: debug use only
	void OutputSite();								//output sites in simple form
	
	//function on marginal distribution of A and w
	void SetZerosMD(int**,int*,int*,int**,int*);				
	void RecordMD(int**,int*,int*,int**,int*);
//	void OutputMD(int**,int*,int*,int**,int*,CString);
	void OutputModPr(int**,int*,int,CString);
	void DeleteMD(int**,int*,int*,int**);
	void SumNeighborMD(int**);
	void FindOptimum(int**,int*,int,bool);
	void AlignSites(int**);
	void InferSiteCount(int*);
	void InferModule(int**,int*);


	//*************friend function
	friend int		Maximum(double*,int);		//find the index for the maximum of a vector
	friend int		Minimum(int*,int);		//minimal of the vector
	friend int Sum(int*,int);				//calculate the sum of the vectors
	friend double Maximum(double,double);		//get the maximum of the two numbers
	friend int		WordCode(CString);			//code the word
	friend CString	DeCodeWord(int,int);		//decode the word
	friend CString Reverse(CString);					//reverse the sequence
	friend int intpow(int,int);						//calculate int power
	friend double LogSum(double,double);		//calculate log(exp(x1)+exp(x2));
	friend double logGamma(int);				//calculate log gamma function
	friend CString ReadData(CString,int,int,double*);	

};

