#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "MyString.h"

///////////////////////////////////////////////////////////////
//Construction/destruction
CString::CString()
{
	m_nstrLength=0;
	m_pString=NULL;
}

CString::CString(const CString& string)
{
	m_nstrLength=string.m_nstrLength;
	m_pString=new char[m_nstrLength+1];
	strcpy(m_pString,string.m_pString);
}

CString::CString(char letter)
{
	m_nstrLength=1;
	m_pString=new char[2];
	m_pString[0]=letter;
	m_pString[1]=0;
}

CString::~CString()
{
	if(m_pString!=NULL)
	{
		delete []m_pString;
		m_pString=NULL;
	}
}

/////////////////////////////////////////////////////////////
//operators
CString CString::operator +(const CString& string)
{
	CString strResult;
	strResult.m_nstrLength=m_nstrLength+string.m_nstrLength;
	strResult.m_pString=new char[strResult.m_nstrLength+1];
	if(m_pString!=NULL)
	{
		strcpy(strResult.m_pString,m_pString);
		strcat(strResult.m_pString,string.m_pString);
	}
	else 
		strcpy(strResult.m_pString,string.m_pString);
	return strResult;
}

CString CString::operator +(const char* pcharstring)
{
	CString strResult;
	strResult.m_nstrLength=m_nstrLength+strlen(pcharstring);
	strResult.m_pString=new char[strResult.m_nstrLength+1];
	if(m_pString!=NULL)
	{
		strcpy(strResult.m_pString,m_pString);
		strcat(strResult.m_pString,pcharstring);
	}
	else
		strcpy(strResult.m_pString,pcharstring);

	return strResult;
}

const CString& CString::operator =(const CString& string)
{
	if(this!=&string)
	{
		m_nstrLength=string.m_nstrLength;
		if(m_pString!=NULL) delete[] m_pString;
		m_pString=new char[m_nstrLength+1];
		strcpy(m_pString,string.m_pString);
	}
	return *this;
}

const CString& CString::operator =(const char* pcharstring)
{
	m_nstrLength=strlen(pcharstring);
	if(m_pString!=NULL) delete[] m_pString;
	m_pString=new char[m_nstrLength+1];
	strcpy(m_pString,pcharstring);
	return *this;
}

const CString& CString::operator =(char letter)
{
	m_nstrLength=1;
	if(m_pString!=NULL) delete[] m_pString;
	m_pString=new char[m_nstrLength+1];
	m_pString[0]=letter;
	m_pString[1]=0;
	return *this;
}

bool CString::operator ==(const CString& string)
{
	if(m_nstrLength==string.m_nstrLength)
	{
		for(int i=0;i<m_nstrLength;i++)
		{
			if(m_pString[i]!=string.m_pString[i]) return false;
		}
		return true;
	}
	else
		return false;
}


char CString::operator [](int nIndex)
{
	return m_pString[nIndex];
}


/////////////////////////////////////////////////////////////
//operators

//get the length of the string class
int CString::GetLength()
{
	return m_nstrLength;
}

//get the left ncount char to form a new string
CString CString::Left(int nCount)
{
	if(nCount>m_nstrLength) nCount=m_nstrLength;
	CString strLeft;
	strLeft.m_nstrLength=nCount;
	strLeft.m_pString=new char[nCount+1];
	for(int i=0;i<nCount;i++)
		strLeft.m_pString[i]=m_pString[i];
	strLeft.m_pString[nCount]=m_pString[m_nstrLength];
	return strLeft;
}

//get the right ncount char to form a new string
CString CString::Right(int nCount)
{
	if(nCount>m_nstrLength) nCount=m_nstrLength;
	CString strRight;
	strRight.m_nstrLength=nCount;
	strRight.m_pString=new char[nCount+1];
	for(int i=0;i<nCount;i++)
		strRight.m_pString[i]=m_pString[i+m_nstrLength-nCount];
	strRight.m_pString[nCount]=m_pString[m_nstrLength];
	return strRight;
}

//get the middle parts of a string
CString CString::Mid(int nStart,int nCount)
{
	if(nStart<0) nStart=0;
	if(nStart>=m_nstrLength) nStart=m_nstrLength-1;
	if(nCount+nStart>m_nstrLength) 
	{
		return *this;
	}
	CString strMid;
	strMid.m_nstrLength=nCount;
	strMid.m_pString=new char[nCount+1];
	for(int i=0;i<nCount;i++)
	{
		strMid.m_pString[i]=m_pString[nStart+i];
	}
	strMid.m_pString[nCount]=m_pString[m_nstrLength];
	return strMid;
}

void CString::Mid(int nStart,int nCount,CString* pstrMid)
{
	if(pstrMid->m_pString!=NULL) delete[] pstrMid->m_pString;
	pstrMid->m_nstrLength=nCount;
	pstrMid->m_pString=new char[nCount+1];
	for(int i=0;i<nCount;i++)
	{
		pstrMid->m_pString[i]=m_pString[nStart+i];
	}
	pstrMid->m_pString[nCount]=m_pString[m_nstrLength];
}


void CString::SetAt(int nIndex,char letter)
{
	if(nIndex>=0 && nIndex<m_nstrLength)
	{
		m_pString[nIndex]=letter;
	}
}

int CString::Find(const CString& string,int nStart)
{
	if(nStart+string.m_nstrLength>m_nstrLength)
		return -1;
	int nScanCount=m_nstrLength-string.m_nstrLength+1-nStart;
	for(int i=0;i<nScanCount;i++)
	{
		if(Mid(i+nStart,string.m_nstrLength)==string)
		{
			return (i+nStart);
		}
	}
	return -1;
}

int CString::Find(const CString& string)
{
	return Find(string,0);
}

int CString::Find(const char* pchar,int nStart)
{
	CString string;
	string=pchar;
	return Find(string,nStart);
}

int CString::Find(const char* pchar)
{
	return Find(pchar,0);
}
