//////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include "StochasticDistribution.h"


#define	PI	3.14159265


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStochasticDistribution::CStochasticDistribution()
{

}

CStochasticDistribution::~CStochasticDistribution()
{

}

double CStochasticDistribution::Uniform(double dblLow, double dblUp)	//generate uniform distribution
{
	return(dblLow+(dblUp-dblLow)*double(rand())/(RAND_MAX+1.0));
}

double CStochasticDistribution::Normal(double dblExpectation, double dblSigma)		//generate normal distribution
{
	double dblOutput=0;
	//generate normal distribution by Box and Muller method
	double U=double(rand()+1.0)/(RAND_MAX+2.0);
	double V=double(rand()+1.0)/(RAND_MAX+2.0);
	double seta=2*PI*U;
	double R=sqrt(-2*log(V));
	dblOutput=R*cos(seta);

	return (dblSigma*dblOutput+dblExpectation);
}

double CStochasticDistribution::Beta(int na,int nb)
{
	double dblChi2_a=0;
	double dblU;
	int j=0;
	for(j=0;j<na;j++)
	{
		dblU=double(rand()+1.0)/(RAND_MAX+2.0);
		dblChi2_a+=log(dblU);
	}

	double dblChi2_b=0;
	for(j=0;j<nb;j++)
	{
		dblU=double(rand()+1.0)/(RAND_MAX+2.0);
		dblChi2_b+=log(dblU);
	}
	 
	double dblBeta=dblChi2_a/(dblChi2_a+dblChi2_b);
	return dblBeta;
}

void CStochasticDistribution::Dirichlet(int* pn,int nDimension,double* pDirchilet)
{
	double dblU;
	double dblSum=0;
	int d=0;
	for(d=0;d<nDimension;d++)
	{
		pDirchilet[d]=0;
		for(int j=0;j<pn[d];j++)
		{
			dblU=double(rand()+1.0)/(RAND_MAX+2.0);
			pDirchilet[d]+=log(dblU);
		}
		dblSum+=pDirchilet[d];
	}
	 
	for(d=0;d<nDimension;d++)
	{
		pDirchilet[d]/=dblSum;
	}
}

//discrete sampling
int	CStochasticDistribution::Discrete(double* dblPrWeight,int nTotal)
{
	int i=0;
	for(i=1;i<nTotal;i++)
		dblPrWeight[i]=LogSum(dblPrWeight[i-1],dblPrWeight[i]);

	double dblTotSum=dblPrWeight[nTotal-1];

	for(i=0;i<nTotal;i++)
	{
		dblPrWeight[i]-=dblTotSum;
		if(dblPrWeight[i]>-300)
			dblPrWeight[i]=exp(dblPrWeight[i]);
		else
			dblPrWeight[i]=0;

		double dblbound=1-1e-10;
		if(dblPrWeight[i]>dblbound)
			dblPrWeight[i]=1.01;
	}
	dblPrWeight[nTotal-1]=1.01;

	//sample:
	double dblU=double(rand()+1.0)/(RAND_MAX+2.0);

	int nIndex=-1;
	for(i=0;i<nTotal;i++)
	{
		if(dblU<dblPrWeight[i])
		{
			nIndex=i;
			break;
		}
	}

	return nIndex;
}

double LogSum(double dblLogP1,double dblLogP2)
{
	//let x1>=x2
	if(dblLogP1<dblLogP2)
	{
		double dblTemp=dblLogP1;
		dblLogP1=dblLogP2;
		dblLogP2=dblTemp;
	}

	double dblLogDiff=dblLogP2-dblLogP1;
	double dblLogSum=dblLogP1;
	if(dblLogDiff>-300)
	{
		dblLogSum+=log(1+exp(dblLogDiff));
	}
	return dblLogSum;
}
